from enum import Enum


class AtlasEventTypes(Enum):
    APP_SETTINGS_CHANGE = 'App Settings Change'
    PEER_CREATED = 'Peer Created'
    PEER_DELETED = 'Peer Deleted'
    PEER_UPDATED = 'Peer Updated'
    GCP_PEER_CREATED = 'GCP Peer Created'
    GCP_PEER_DELETED = 'GCP Peer Deleted'
    GCP_PEER_UPDATED = 'GCP Peer Updated'
    GCP_PEER_ACTIVE = 'GCP Peer Active'
    GCP_PEER_INACTIVE = 'GCP Peer Inactive'
    AZURE_PEER_CREATED = 'Azure Peer Created'
    AZURE_PEER_UPDATED = 'Azure Peer Updated'
    AZURE_PEER_ACTIVE = 'Azure Peer Active'
    AZURE_PEER_DELETED = 'Azure Peer Deleted'
    NEW_AGENT = 'New Agent'
    MONITORING_AGENT_UP = 'Monitoring Agent Up'
    MONITORING_AGENT_DOWN = 'Monitoring Agent Down'
    BACKUP_AGENT_UP = 'Backup Agent Up'
    BACKUP_AGENT_DOWN = 'Backup Agent Down'
    AUTOMATION_AGENT_UP = 'Automation Agent Up'
    AUTOMATION_AGENT_DOWN = 'Automation Agent Down'
    NDS_PROXY_UP = 'Nds Proxy Up'
    NDS_PROXY_DOWN = 'Nds Proxy Down'
    MONITORING_AGENT_VERSION_CURRENT = 'Monitoring Agent Version Current'
    MONITORING_AGENT_VERSION_BEHIND = 'Monitoring Agent Version Behind'
    BACKUP_AGENT_VERSION_CURRENT = 'Backup Agent Version Current'
    BACKUP_AGENT_VERSION_BEHIND = 'Backup Agent Version Behind'
    BACKUP_AGENT_CONF_CALL_OK = 'Backup Agent Conf Call Ok'
    BACKUP_AGENT_CONF_CALL_FAILURE = 'Backup Agent Conf Call Failure'
    ALERT_ACKNOWLEDGED_AUDIT = 'Alert Acknowledged Audit'
    ALERT_UNACKNOWLEDGED_AUDIT = 'Alert Unacknowledged Audit'
    ALERT_CONFIG_DISABLED_AUDIT = 'Alert Config Disabled Audit'
    ALERT_CONFIG_ENABLED_AUDIT = 'Alert Config Enabled Audit'
    ALERT_CONFIG_ADDED_AUDIT = 'Alert Config Added Audit'
    ALERT_CONFIG_DELETED_AUDIT = 'Alert Config Deleted Audit'
    ALERT_CONFIG_CHANGED_AUDIT = 'Alert Config Changed Audit'
    AUTOMATION_CONFIG_PUBLISHED_AUDIT = 'Automation Config Published Audit'
    OPLOG_CURRENT = 'Oplog Current'
    OPLOG_BEHIND = 'Oplog Behind'
    RESYNC_PERFORMED = 'Resync Performed'
    RESYNC_REQUIRED = 'Resync Required'
    BACKUP_RECOVERED = 'Backup Recovered'
    BACKUP_IN_UNEXPECTED_STATE = 'Backup In Unexpected State'
    BACKUP_TOO_MANY_RETRIES = 'Backup Too Many Retries'
    BACKUP_TOO_MANY_HEAD_START_ATTEMPTS = 'Backup Too Many Head Start Attempts'
    BLOCKSTORE_JOB_RECOVERED = 'Blockstore Job Recovered'
    BLOCKSTORE_JOB_TOO_MANY_RETRIES = 'Blockstore Job Too Many Retries'
    NO_RS_BIND_ERROR = 'No Rs Bind Error'
    RS_BIND_ERROR = 'Rs Bind Error'
    TIMELY_SNAPSHOT = 'Timely Snapshot'
    LATE_SNAPSHOT = 'Late Snapshot'
    GOOD_CLUSTERSHOT = 'Good Clustershot'
    BAD_CLUSTERSHOTS = 'Bad Clustershots'
    SYNC_SLICE_PROGRESSED = 'Sync Slice Progressed'
    SYNC_SLICE_HAS_NOT_PROGRESSED = 'Sync Slice Has Not Progressed'
    BACKUP_JOB_NOT_BUSY = 'Backup Job Not Busy'
    BACKUP_JOB_TOO_BUSY = 'Backup Job Too Busy'
    CONSISTENT_BACKUP_CONFIGURATION = 'Consistent Backup Configuration'
    INCONSISTENT_BACKUP_CONFIGURATION = 'Inconsistent Backup Configuration'
    DAEMON_AVAILABLE_FOR_QUERYABLE_RESTORE_JOB = 'Daemon Available For Queryable Restore Job'
    NO_DAEMON_AVAILABLE_FOR_QUERYABLE_RESTORE_JOB = 'No Daemon Available For Queryable Restore Job'
    INITIAL_SYNC_STARTED_AUDIT = 'Initial Sync Started Audit'
    INITIAL_SYNC_FINISHED_AUDIT = 'Initial Sync Finished Audit'
    RS_STATE_CHANGED_AUDIT = 'Rs State Changed Audit'
    CLUSTER_STATE_CHANGED_AUDIT = 'Cluster State Changed Audit'
    RESTORE_REQUESTED_AUDIT = 'Restore Requested Audit'
    INTERNAL_DIAGNOSTIC_RESTORE_REQUESTED_AUDIT = 'Internal Diagnostic Restore Requested Audit'
    SYNC_REQUIRED_AUDIT = 'Sync Required Audit'
    SYNC_PENDING_AUDIT = 'Sync Pending Audit'
    CLUSTERSHOT_DELETED_AUDIT = 'Clustershot Deleted Audit'
    SNAPSHOT_DELETED_AUDIT = 'Snapshot Deleted Audit'
    RS_CREDENTIAL_UPDATED_AUDIT = 'Rs Credential Updated Audit'
    CLUSTER_CREDENTIAL_UPDATED_AUDIT = 'Cluster Credential Updated Audit'
    RS_BLACKLIST_UPDATED_AUDIT = 'Rs Blacklist Updated Audit'
    CLUSTER_BLACKLIST_UPDATED_AUDIT = 'Cluster Blacklist Updated Audit'
    RS_SNAPSHOT_SCHEDULE_UPDATED_AUDIT = 'Rs Snapshot Schedule Updated Audit'
    CLUSTER_SNAPSHOT_SCHEDULE_UPDATED_AUDIT = 'Cluster Snapshot Schedule Updated Audit'
    CLUSTER_CHECKPOINT_UPDATED_AUDIT = 'Cluster Checkpoint Updated Audit'
    RS_STORAGE_ENGINE_UPDATED_AUDIT = 'Rs Storage Engine Updated Audit'
    CLUSTER_STORAGE_ENGINE_UPDATED_AUDIT = 'Cluster Storage Engine Updated Audit'
    RS_ROTATE_MASTER_KEY_AUDIT = 'Rs Rotate Master Key Audit'
    SNAPSHOT_EXPIRY_UPDATED_AUDIT = 'Snapshot Expiry Updated Audit'
    CLUSTERSHOT_EXPIRY_UPDATED_AUDIT = 'Clustershot Expiry Updated Audit'
    CPS_RESTORE_REQUESTED_AUDIT = 'Cps Restore Requested Audit'
    CPS_SNAPSHOT_SCHEDULE_UPDATED_AUDIT = 'Cps Snapshot Schedule Updated Audit'
    CPS_SNAPSHOT_DELETED_AUDIT = 'Cps Snapshot Deleted Audit'
    CPS_SNAPSHOT_RETENTION_MODIFIED_AUDIT = 'Cps Snapshot Retention Modified Audit'
    CPS_SNAPSHOT_IN_PROGRESS_AUDIT = 'Cps Snapshot In Progress Audit'
    CPS_SNAPSHOT_COMPLETED_AUDIT = 'Cps Snapshot Completed Audit'
    CPS_ON_DEMAND_SNAPSHOT_REQUESTED = 'Cps On Demand Snapshot Requested'
    BUCKET_CREATED_AUDIT = 'Bucket Created Audit'
    BUCKET_DELETED_AUDIT = 'Bucket Deleted Audit'
    TENANT_SNAPSHOT_STARTED_AUDIT = 'Tenant Snapshot Started Audit'
    TENANT_SNAPSHOT_COMPLETED_AUDIT = 'Tenant Snapshot Completed Audit'
    TENANT_SNAPSHOT_DELETED_AUDIT = 'Tenant Snapshot Deleted Audit'
    TENANT_RESTORE_REQUESTED_AUDIT = 'Tenant Restore Requested Audit'
    TENANT_RESTORE_COMPLETED_AUDIT = 'Tenant Restore Completed Audit'
    TENANT_SNAPSHOT_DOWNLOAD_REQUESTED_AUDIT = 'Tenant Snapshot Download Requested Audit'
    BI_CONNECTOR_DOWN = 'Bi Connector Down'
    BI_CONNECTOR_UP = 'Bi Connector Up'
    CHARGE_SUCCEEDED = 'Charge Succeeded'
    CHARGE_FAILED = 'Charge Failed'
    BRAINTREE_CHARGE_FAILED = 'Braintree Charge Failed'
    CHECK_PAYMENT_RECEIVED = 'Check Payment Received'
    WIRE_TRANSFER_PAYMENT_RECEIVED = 'Wire Transfer Payment Received'
    INVOICE_CLOSED = 'Invoice Closed'
    CREDIT_CARD_CURRENT = 'Credit Card Current'
    CREDIT_CARD_ABOUT_TO_EXPIRE = 'Credit Card About To Expire'
    NO_STALE_PENDING_INVOICES = 'No Stale Pending Invoices'
    STALE_PENDING_INVOICES = 'Stale Pending Invoices'
    INVOICE_BILLED_EQUALS_LINE_ITEMS_TOTAL = 'Invoice Billed Equals Line Items Total'
    INVOICE_BILLED_DOES_NOT_EQUAL_LINE_ITEMS_TOTAL = 'Invoice Billed Does Not Equal Line Items Total'
    ONE_PENDING_INVOICE = 'One Pending Invoice'
    TOO_MANY_PENDING_INVOICES = 'Too Many Pending Invoices'
    PENDING_INVOICE_UNDER_THRESHOLD = 'Pending Invoice Under Threshold'
    PENDING_INVOICE_OVER_THRESHOLD = 'Pending Invoice Over Threshold'
    DAILY_BILL_UNDER_THRESHOLD = 'Daily Bill Under Threshold'
    DAILY_BILL_OVER_THRESHOLD = 'Daily Bill Over Threshold'
    PAYMENT_METHOD_ADDED = 'Payment Method Added'
    MISSING_PAYMENT_METHOD = 'Missing Payment Method'
    PREPAID_PLAN_NOT_MISSING_SKU = 'Prepaid Plan Not Missing Sku'
    PREPAID_PLAN_MISSING_SKU = 'Prepaid Plan Missing Sku'
    PREPAID_PLAN_ACTIVATED = 'Prepaid Plan Activated'
    PAYMENT_UPDATED_THROUGH_API = 'Payment Updated Through API'
    NO_NONZERO_PENDING_REFUNDS = 'No Nonzero Pending Refunds'
    NONZERO_PENDING_REFUNDS = 'Nonzero Pending Refunds'
    NO_DUPLICATE_SUBSCRIPTION_USAGE = 'No Duplicate Subscription Usage'
    DUPLICATE_SUBSCRIPTION_USAGE = 'Duplicate Subscription Usage'
    INVOICE_BILLED_EQUALS_PAID = 'Invoice Billed Equals Paid'
    INVOICE_BILLED_DOES_NOT_EQUAL_PAID = 'Invoice Billed Does Not Equal Paid'
    BILLABLE_HOSTS_INCREASED = 'Billable Hosts Increased'
    ONE_INVOICE_FOR_MONTH = 'One Invoice For Month'
    MULTIPLE_INVOICES_FOR_MONTH = 'Multiple Invoices For Month'
    PAYMENT_METHODS_REMOVED = 'Payment Methods Removed'
    DISCOUNT_APPLIED = 'Discount Applied'
    CREDIT_ISSUED = 'Credit Issued'
    GCP_BILLING_ACCOUNT_CREDIT_ISSUED = 'GCP Billing Account Credit Issued'
    CREDIT_PULLED_FWD = 'Credit Pulled Fwd'
    CREDIT_END_DATE_MODIFIED = 'Credit End Date Modified'
    CREDIT_SFOLID_MODIFIED = 'Credit Sfolid Modified'
    CREDIT_AMOUNT_MODIFIED = 'Credit Amount Modified'
    PROMO_CODE_APPLIED = 'Promo Code Applied'
    REFUND_ISSUED = 'Refund Issued'
    PAYMENT_FORGIVEN = 'Payment Forgiven'
    ACCOUNT_UPGRADED = 'Account Upgraded'
    ACCOUNT_DOWNGRADED = 'Account Downgraded'
    SUPPORT_PLAN_ACTIVATED = 'Support Plan Activated'
    SUPPORT_PLAN_CANCELLED = 'Support Plan Cancelled'
    SUPPORT_PLAN_CANCELLATION_SCHEDULED = 'Support Plan Cancellation Scheduled'
    ACCOUNT_MODIFIED = 'Account Modified'
    INVOICE_ADDRESS_ADDED = 'Invoice Address Added'
    INVOICE_ADDRESS_CHANGED = 'Invoice Address Changed'
    ELASTIC_INVOICING_MODE_ACTIVATED = 'Elastic Invoicing Mode Activated'
    ELASTIC_INVOICING_MODE_DEACTIVATED = 'Elastic Invoicing Mode Deactivated'
    BECAME_PAYING_ORG = 'Became Paying Org'
    NEW_LINKED_ORG = 'New Linked Org'
    ORG_LINKED_TO_PAYING_ORG = 'Org Linked To Paying Org'
    METER_USAGE_DELETED_THROUGH_API = 'Meter Usage Deleted Through API'
    TERMINATE_PAID_SERVICES = 'Terminate Paid Services'
    CLUSTER_MONGOS_IS_PRESENT = 'Cluster Mongos Is Present'
    CLUSTER_MONGOS_IS_MISSING = 'Cluster Mongos Is Missing'
    SHARD_ADDED = 'Shard Added'
    SHARD_REMOVED = 'Shard Removed'
    DATA_EXPLORER = 'Data Explorer'
    DATA_EXPLORER_CRUD = 'Data Explorer Crud'
    DATA_EXPLORER_CRUD_ERROR = 'Data Explorer Crud Error'
    DATA_EXPLORER_CRUD_ATTEMPT = 'Data Explorer Crud Attempt'
    AWS_ENCRYPTION_KEY_ROTATED = 'AWS Encryption Key Rotated'
    AWS_ENCRYPTION_KEY_NEEDS_ROTATION = 'AWS Encryption Key Needs Rotation'
    AZURE_ENCRYPTION_KEY_ROTATED = 'Azure Encryption Key Rotated'
    AZURE_ENCRYPTION_KEY_NEEDS_ROTATION = 'Azure Encryption Key Needs Rotation'
    GCP_ENCRYPTION_KEY_ROTATED = 'GCP Encryption Key Rotated'
    GCP_ENCRYPTION_KEY_NEEDS_ROTATION = 'GCP Encryption Key Needs Rotation'
    CREDIT_CARD_ADDED = 'Credit Card Added'
    CREDIT_CARD_UPDATED = 'Credit Card Updated'
    GROUP_DELETED = 'Group Deleted'
    GROUP_CREATED = 'Group Created'
    GROUP_MOVED = 'Group Moved'
    GROUP_TEMPORARILY_ACTIVATED = 'Group Temporarily Activated'
    GROUP_ACTIVATED = 'Group Activated'
    GROUP_LOCKED = 'Group Locked'
    GROUP_SUSPENDED = 'Group Suspended'
    GROUP_FLUSHED = 'Group Flushed'
    GROUP_NAME_CHANGED = 'Group Name Changed'
    GROUP_TAGS_CHANGED = 'Group Tags Changed'
    GROUP_CHARTS_ACTIVATION_REQUESTED = 'Group Charts Activation Requested'
    GROUP_CHARTS_ACTIVATED = 'Group Charts Activated'
    GROUP_CHARTS_UPGRADED = 'Group Charts Upgraded'
    GROUP_CHARTS_RESET = 'Group Charts Reset'
    LICENSE_ACCEPTED = 'License Accepted'
    ACCOUNT_CLOSED = 'Account Closed'
    COMPANY_NAME_OFAC_HIT = 'Company Name OFAC Hit'
    GROUP_UNEMBARGOED = 'Group Unembargoed'
    AGENT_API_KEY_CREATED = 'Agent API Key Created'
    AGENT_API_KEY_DELETED = 'Agent API Key Deleted'
    PAID_IN_FULL = 'Paid In Full'
    DELINQUENT = 'Delinquent'
    NO_USERS_AWAITING_APPROVAL = 'No Users Awaiting Approval'
    USERS_AWAITING_APPROVAL = 'Users Awaiting Approval'
    ALL_USERS_HAVE_MULTI_FACTOR_AUTH = 'All Users Have Multi Factor Auth'
    USERS_WITHOUT_MULTI_FACTOR_AUTH = 'Users Without Multi Factor Auth'
    SECURITY_CHECKUP_ALERTS_UPDATED = 'Security Checkup Alerts Updated'
    NEW_HOST = 'New Host'
    HOST_RESTARTED = 'Host Restarted'
    HOST_ROLLBACK = 'Host Rollback'
    HOST_UPGRADED = 'Host Upgraded'
    HOST_DOWNGRADED = 'Host Downgraded'
    HOST_VERSION_CHANGED = 'Host Version Changed'
    HOST_NOW_PRIMARY = 'Host Now Primary'
    HOST_NOW_SECONDARY = 'Host Now Secondary'
    HOST_NOW_STANDALONE = 'Host Now Standalone'
    HOST_UP = 'Host Up'
    HOST_DOWN = 'Host Down'
    HOST_HAS_INDEX_SUGGESTIONS = 'Host Has Index Suggestions'
    HOST_HAS_INDEX_SUGGESTIONS_WITHDRAWN = 'Host Has Index Suggestions Withdrawn'
    HOST_RECOVERED = 'Host Recovered'
    HOST_RECOVERING = 'Host Recovering'
    HOST_VERSION_CURRENT = 'Host Version Current'
    HOST_VERSION_BEHIND = 'Host Version Behind'
    HOST_SSL_CERTIFICATE_CURRENT = 'Host SSL Certificate Current'
    HOST_SSL_CERTIFICATE_STALE = 'Host SSL Certificate Stale'
    HOST_SECURITY_CHECKUP_MET = 'Host Security Checkup Met'
    HOST_SECURITY_CHECKUP_NOT_MET = 'Host Security Checkup Not Met'
    INSIDE_METRIC_THRESHOLD = 'Inside Metric Threshold'
    OUTSIDE_METRIC_THRESHOLD = 'Outside Metric Threshold'
    ANOMALOUS_VALUE_DETECTED = 'Anomalous Value Detected'
    VALUE_NO_LONGER_ANOMALOUS = 'Value No Longer Anomalous'
    HOST_LOCKED_DOWN = 'Host Locked Down'
    HOST_EXPOSED = 'Host Exposed'
    DELETE_HOST_AUDIT = 'Delete Host Audit'
    REACTIVATE_HOST_AUDIT = 'Reactivate Host Audit'
    DEACTIVATE_HOST_AUDIT = 'Deactivate Host Audit'
    ADD_HOST_AUDIT = 'Add Host Audit'
    SSH_KEY_NDS_HOST_ACCESS_REQUESTED = 'Ssh Key Nds Host Access Requested'
    SSH_KEY_NDS_HOST_ACCESS_REFRESHED = 'Ssh Key Nds Host Access Refreshed'
    SSH_KEY_NDS_HOST_ACCESS_ATTEMPT = 'Ssh Key Nds Host Access Attempt'
    SSH_KEY_NDS_HOST_ACCESS_GRANTED = 'Ssh Key Nds Host Access Granted'
    NDS_HOST_LOGS_DOWNLOADED = 'Nds Host Logs Downloaded'
    NDS_CLOUD_PROVIDER_CONSOLE_INFORMATION_DOWNLOADED = 'Nds Cloud Provider Console Information Downloaded'
    UNDELETE_HOST_AUDIT = 'Undelete Host Audit'
    HIDE_HOST_AUDIT = 'Hide Host Audit'
    HIDE_AND_DISABLE_HOST_AUDIT = 'Hide And Disable Host Audit'
    DISABLE_HOST_AUDIT = 'Disable Host Audit'
    PAUSE_HOST_AUDIT = 'Pause Host Audit'
    RESUME_HOST_AUDIT = 'Resume Host Audit'
    ADD_HOST_TO_REPLICA_SET_AUDIT = 'Add Host To Replica Set Audit'
    REMOVE_HOST_FROM_REPLICA_SET_AUDIT = 'Remove Host From Replica Set Audit'
    DB_PROFILER_ENABLE_AUDIT = 'Db Profiler Enable Audit'
    DB_PROFILER_DISABLE_AUDIT = 'Db Profiler Disable Audit'
    HOST_IP_CHANGED_AUDIT = 'Host IP Changed Audit'
    AUTO_CREATED_INDEX_AUDIT = 'Auto Created Index Audit'
    ATTEMPT_KILLOP_AUDIT = 'Attempt Killop Audit'
    ATTEMPT_KILLSESSION_AUDIT = 'Attempt Killsession Audit'
    LOG_COLLECTION_REQUESTED = 'Log Collection Requested'
    LOG_DOWNLOAD_STARTED = 'Log Download Started'
    MAINTENANCE_WINDOW_ADDED_AUDIT = 'Maintenance Window Added Audit'
    MAINTENANCE_WINDOW_CHANGED_AUDIT = 'Maintenance Window Changed Audit'
    MAINTENANCE_WINDOW_DELETED_AUDIT = 'Maintenance Window Deleted Audit'
    CLUSTER_CREATED = 'Cluster Created'
    CLUSTER_READY = 'Cluster Ready'
    CLUSTER_UPDATE_SUBMITTED = 'Cluster Update Submitted'
    CLUSTER_UPDATE_STARTED = 'Cluster Update Started'
    CLUSTER_UPDATE_COMPLETED = 'Cluster Update Completed'
    CLUSTER_PROCESS_ARGS_UPDATE_SUBMITTED = 'Cluster Process Args Update Submitted'
    CLUSTER_AUTOMATICALLY_PAUSED = 'Cluster Automatically Paused'
    CLUSTER_DELETE_SUBMITTED = 'Cluster Delete Submitted'
    CLUSTER_DELETED = 'Cluster Deleted'
    CLUSTER_IMPORT_STARTED = 'Cluster Import Started'
    CLUSTER_IMPORT_ACKNOWLEDGED = 'Cluster Import Acknowledged'
    CLUSTER_IMPORT_CANCELLED = 'Cluster Import Cancelled'
    CLUSTER_IMPORT_EXPIRED = 'Cluster Import Expired'
    CLUSTER_IMPORT_EXTENDED = 'Cluster Import Extended'
    CLUSTER_IMPORT_CUTOVER = 'Cluster Import Cutover'
    CLUSTER_IMPORT_VALIDATION_SUCCESS = 'Cluster Import Validation Success'
    CLUSTER_IMPORT_VALIDATION_FAIL = 'Cluster Import Validation Fail'
    CLUSTER_IMPORT_RESTART_REQUESTED = 'Cluster Import Restart Requested'
    GLOBAL_SOFTWARE_TYPE_UPDATED = 'Global Software Type Updated'
    CLUSTER_VERSION_FIXED = 'Cluster Version Fixed'
    CLUSTER_VERSION_UNFIXED = 'Cluster Version Unfixed'
    CLUSTER_FCV_FIXED = 'Cluster Fcv Fixed'
    CLUSTER_FCV_UNFIXED = 'Cluster Fcv Unfixed'
    CLUSTER_OPLOG_RESIZED = 'Cluster Oplog Resized'
    CLUSTER_INSTANCE_RESTARTED = 'Cluster Instance Restarted'
    CLUSTER_INSTANCE_REPLACED = 'Cluster Instance Replaced'
    CLUSTER_INSTANCE_REPLACE_CLEARED = 'Cluster Instance Replace Cleared'
    CLUSTER_INSTANCE_STOP_START = 'Cluster Instance Stop Start'
    CLUSTER_INSTANCE_RESYNC_REQUESTED = 'Cluster Instance Resync Requested'
    CLUSTER_INSTANCE_RESYNC_CLEARED = 'Cluster Instance Resync Cleared'
    CLUSTER_INSTANCE_UPDATE_REQUESTED = 'Cluster Instance Update Requested'
    CLUSTER_CONNECTION_GET_DATABASES = 'Cluster Connection Get Databases'
    CLUSTER_CONNECTION_GET_DATABASE_COLLECTIONS = 'Cluster Connection Get Database Collections'
    CLUSTER_CONNECTION_GET_DATABASE_NAMESPACES = 'Cluster Connection Get Database Namespaces'
    CLUSTER_CONNECTION_AGGREGATE = 'Cluster Connection Aggregate'
    SERVERLESS_INSTANCE_CREATED = 'Serverless Instance Created'
    SERVERLESS_INSTANCE_READY = 'Serverless Instance Ready'
    SERVERLESS_INSTANCE_UPDATE_SUBMITTED = 'Serverless Instance Update Submitted'
    SERVERLESS_INSTANCE_UPDATE_STARTED = 'Serverless Instance Update Started'
    SERVERLESS_INSTANCE_UPDATE_COMPLETED = 'Serverless Instance Update Completed'
    SERVERLESS_INSTANCE_DELETE_SUBMITTED = 'Serverless Instance Delete Submitted'
    SERVERLESS_INSTANCE_DELETED = 'Serverless Instance Deleted'
    MONGODB_LOGS_DOWNLOADED = 'Mongodb Logs Downloaded'
    MONGOSQLD_LOGS_DOWNLOADED = 'Mongosqld Logs Downloaded'
    MONGODB_USER_ADDED = 'Mongodb User Added'
    MONGODB_USER_DELETED = 'Mongodb User Deleted'
    MONGODB_USER_UPDATED = 'Mongodb User Updated'
    MONGODB_ROLE_ADDED = 'Mongodb Role Added'
    MONGODB_ROLE_UPDATED = 'Mongodb Role Updated'
    MONGODB_ROLE_DELETED = 'Mongodb Role Deleted'
    NETWORK_PERMISSION_ENTRY_ADDED = 'Network Permission Entry Added'
    NETWORK_PERMISSION_ENTRY_REMOVED = 'Network Permission Entry Removed'
    NETWORK_PERMISSION_ENTRY_UPDATED = 'Network Permission Entry Updated'
    PLANNING_FAILURE = 'Planning Failure'
    PLAN_STARTED = 'Plan Started'
    PLAN_COMPLETED = 'Plan Completed'
    PLAN_FAILURE = 'Plan Failure'
    PLAN_ABANDONED = 'Plan Abandoned'
    PLAN_FAILURE_COUNT_RESET = 'Plan Failure Count Reset'
    PLAN_ASAP_REQUESTED = 'Plan Asap Requested'
    PROJECT_LIMIT_UPDATED = 'Project Limit Updated'
    MOVE_SKIPPED = 'Move Skipped'
    PROXY_RESTARTED = 'Proxy Restarted'
    PROXY_PANICKED = 'Proxy Panicked'
    TENANT_CLUSTER_UPGRADE_FROM_MTM = 'Tenant Cluster Upgrade From MTM'
    TENANT_SNAPSHOT_FAILED = 'Tenant Snapshot Failed'
    TENANT_RESTORE_FAILED = 'Tenant Restore Failed'
    SCHEDULED_MAINTENANCE = 'Scheduled Maintenance'
    PROJECT_SCHEDULED_MAINTENANCE = 'Project Scheduled Maintenance'
    OS_MAINTENANCE = 'Os Maintenance'
    OS_MAINTENANCE_REPLACEMENT = 'Os Maintenance Replacement'
    ATLAS_MAINTENANCE_WINDOW_ADDED = 'Atlas Maintenance Window Added'
    ATLAS_MAINTENANCE_WINDOW_MODIFIED = 'Atlas Maintenance Window Modified'
    ATLAS_MAINTENANCE_WINDOW_REMOVED = 'Atlas Maintenance Window Removed'
    ATLAS_MAINTENANCE_DEFERRED = 'Atlas Maintenance Deferred'
    ATLAS_MAINTENANCE_AUTO_DEFER_ENABLED = 'Atlas Maintenance Auto Defer Enabled'
    ATLAS_MAINTENANCE_AUTO_DEFER_DISABLED = 'Atlas Maintenance Auto Defer Disabled'
    ATLAS_MAINTENANCE_START_ASAP = 'Atlas Maintenance Start Asap'
    ATLAS_MAINTENANCE_SCHEDULED_FOR_NEXT_WINDOW = 'Atlas Maintenance Scheduled For Next Window'
    COMPUTE_AUTO_SCALE_UNNECESSARY = 'Compute Auto Scale Unnecessary'
    COMPUTE_AUTO_SCALE_TRIGGERED = 'Compute Auto Scale Triggered'
    COMPUTE_AUTO_SCALE_SKIPPED = 'Compute Auto Scale Skipped'
    COMPUTE_AUTO_SCALE_INITIATED = 'Compute Auto Scale Initiated'
    DISK_AUTO_SCALE_INITIATED = 'Disk Auto Scale Initiated'
    FTS_INDEX_DELETION_FAILED = 'Fts Index Deletion Failed'
    FREE_UPGRADE_STARTED = 'Free Upgrade Started'
    TEST_FAILOVER_REQUESTED = 'Test Failover Requested'
    USER_SECURITY_SETTINGS_UPDATED = 'User Security Settings Updated'
    AUDIT_LOG_CONFIGURATION_UPDATED = 'Audit Log Configuration Updated'
    ENCRYPTION_AT_REST_CONFIGURATION_UPDATED = 'Encryption At Rest Configuration Updated'
    ENCRYPTION_AT_REST_CONFIGURATION_VALIDATION_FAILED = 'Encryption At Rest Configuration Validation Failed'
    AGENT_VERSION_FIXED = 'Agent Version Fixed'
    AGENT_VERSION_UNFIXED = 'Agent Version Unfixed'
    CLUSTER_INSTANCE_CONFIG_UPDATED = 'Cluster Instance Config Updated'
    CLUSTER_INSTANCE_SSL_ROTATED = 'Cluster Instance SSL Rotated'
    CLUSTER_INSTANCE_SSL_ROTATED_PER_CLUSTER = 'Cluster Instance SSL Rotated Per Cluster'
    CLUSTER_INSTANCE_SSL_REVOKED = 'Cluster Instance SSL Revoked'
    NDS_SET_IMAGE_OVERRIDES = 'Nds Set Image Overrides'
    NDS_SET_CHEF_TARBALL_URI = 'Nds Set Chef Tarball Uri'
    RESTRICTED_EMPLOYEE_ACCESS_BYPASS = 'Restricted Employee Access Bypass'
    CLOUD_PROVIDER_ACCESS_AWS_IAM_ROLE_ADDED = 'Cloud Provider Access AWS Iam Role Added'
    CLOUD_PROVIDER_ACCESS_AWS_IAM_ROLE_DELETED = 'Cloud Provider Access AWS Iam Role Deleted'
    CLOUD_PROVIDER_ACCESS_AWS_IAM_ROLE_UPDATED = 'Cloud Provider Access AWS Iam Role Updated'
    ORG_CONNECTED_TO_MLAB = 'Org Connected To Mlab'
    ORG_DISCONNECTED_FROM_MLAB = 'Org Disconnected From Mlab'
    MLAB_MIGRATION_INITIATED = 'Mlab Migration Initiated'
    MLAB_MIGRATION_COMPLETED = 'Mlab Migration Completed'
    MLAB_MIGRATION_CANCELLED = 'Mlab Migration Cancelled'
    MLAB_MIGRATION_PREREQUISITES_ACKNOWLEDGED = 'Mlab Migration Prerequisites Acknowledged'
    MLAB_MIGRATION_TARGET_PROJECT_SET = 'Mlab Migration Target Project Set'
    MLAB_MIGRATION_TARGET_CLUSTER_CREATED = 'Mlab Migration Target Cluster Created'
    MLAB_MIGRATION_DATABASE_USERS_IMPORTED = 'Mlab Migration Database Users Imported'
    MLAB_MIGRATION_IP_WHITELIST_IMPORTED = 'Mlab Migration IP Whitelist Imported'
    MLAB_MIGRATION_TARGET_CLUSTER_SET = 'Mlab Migration Target Cluster Set'
    MLAB_MIGRATION_DATABASE_RENAMED = 'Mlab Migration Database Renamed'
    MLAB_MIGRATION_LIVE_IMPORT_STARTED = 'Mlab Migration Live Import Started'
    MLAB_MIGRATION_LIVE_IMPORT_READY_FOR_CUTOVER = 'Mlab Migration Live Import Ready For Cutover'
    MLAB_MIGRATION_LIVE_IMPORT_CUTOVER_COMPLETE = 'Mlab Migration Live Import Cutover Complete'
    MLAB_MIGRATION_LIVE_IMPORT_ERROR = 'Mlab Migration Live Import Error'
    MLAB_MIGRATION_LIVE_IMPORT_CANCELLED = 'Mlab Migration Live Import Cancelled'
    MLAB_MIGRATION_DUMP_AND_RESTORE_TEST_STARTED = 'Mlab Migration Dump And Restore Test Started'
    MLAB_MIGRATION_DUMP_AND_RESTORE_TEST_SKIPPED = 'Mlab Migration Dump And Restore Test Skipped'
    MLAB_MIGRATION_DUMP_AND_RESTORE_STARTED = 'Mlab Migration Dump And Restore Started'
    MLAB_MIGRATION_SUPPORT_PLAN_SELECTED = 'Mlab Migration Support Plan Selected'
    MLAB_MIGRATION_SUPPORT_PLAN_OPTED_OUT = 'Mlab Migration Support Plan Opted Out'
    QUERY_ENGINE_TENANT_CREATED = 'Query Engine Tenant Created'
    QUERY_ENGINE_TENANT_UPDATED = 'Query Engine Tenant Updated'
    QUERY_ENGINE_TENANT_REMOVED = 'Query Engine Tenant Removed'
    X509_USER_CERTIFICATE_GENERATED_BY_APP_INTEGRATION = 'X509 User Certificate Generated By App Integration'
    SAMPLE_DATASET_LOAD_REQUESTED = 'Sample Dataset Load Requested'
    ORG_CREDIT_CARD_ADDED = 'Org Credit Card Added'
    ORG_CREDIT_CARD_UPDATED = 'Org Credit Card Updated'
    ORG_CREDIT_CARD_CURRENT = 'Org Credit Card Current'
    ORG_CREDIT_CARD_ABOUT_TO_EXPIRE = 'Org Credit Card About To Expire'
    ORG_PAYPAL_LINKED = 'Org Paypal Linked'
    ORG_PAYPAL_UPDATED = 'Org Paypal Updated'
    ORG_PAYPAL_CANCELLED = 'Org Paypal Cancelled'
    ORG_ACTIVATED = 'Org Activated'
    ORG_TEMPORARILY_ACTIVATED = 'Org Temporarily Activated'
    ORG_IP_WHITELIST_DELETED = 'Org IP Whitelist Deleted'
    ORG_IP_ACCESS_LIST_DELETED = 'Org IP Access List Deleted'
    ORG_CLUSTERS_DELETED = 'Org Clusters Deleted'
    ORG_CLUSTERS_PAUSED = 'Org Clusters Paused'
    ORG_SUSPENDED = 'Org Suspended'
    ORG_ADMIN_SUSPENDED = 'Org Admin Suspended'
    ORG_LOCKED = 'Org Locked'
    ORG_COMPANY_NAME_OFAC_HIT = 'Org Company Name OFAC Hit'
    ORG_EMBARGO_CONFIRMED = 'Org Embargo Confirmed'
    ORG_UNEMBARGOED = 'Org Unembargoed'
    ORG_CREATED = 'Org Created'
    ORG_DELETED = 'Org Deleted'
    ORG_RENAMED = 'Org Renamed'
    ALL_ORG_USERS_HAVE_MFA = 'All Org Users Have MFA'
    ORG_USERS_WITHOUT_MFA = 'Org Users Without MFA'
    ORG_INVOICE_UNDER_THRESHOLD = 'Org Invoice Under Threshold'
    ORG_INVOICE_OVER_THRESHOLD = 'Org Invoice Over Threshold'
    ORG_DAILY_BILL_UNDER_THRESHOLD = 'Org Daily Bill Under Threshold'
    ORG_DAILY_BILL_OVER_THRESHOLD = 'Org Daily Bill Over Threshold'
    ORG_GROUP_CHARGES_UNDER_THRESHOLD = 'Org Group Charges Under Threshold'
    ORG_GROUP_CHARGES_OVER_THRESHOLD = 'Org Group Charges Over Threshold'
    ORG_TWO_FACTOR_AUTH_REQUIRED = 'Org Two Factor Auth Required'
    ORG_TWO_FACTOR_AUTH_OPTIONAL = 'Org Two Factor Auth Optional'
    ORG_PUBLIC_API_WHITELIST_REQUIRED = 'Org Public API Whitelist Required'
    ORG_PUBLIC_API_WHITELIST_NOT_REQUIRED = 'Org Public API Whitelist Not Required'
    ORG_PUBLIC_API_ACCESS_LIST_REQUIRED = 'Org Public API Access List Required'
    ORG_PUBLIC_API_ACCESS_LIST_NOT_REQUIRED = 'Org Public API Access List Not Required'
    ORG_EMPLOYEE_ACCESS_RESTRICTED = 'Org Employee Access Restricted'
    ORG_EMPLOYEE_ACCESS_UNRESTRICTED = 'Org Employee Access Unrestricted'
    ORG_SFDC_ACCOUNT_ID_CHANGED = 'Org Sfdc Account Id Changed'
    ORG_OVERRIDE_PAYMENT_METHOD_ADDED = 'Org Override Payment Method Added'
    API_KEY_WHITELIST_ENTRY_ADDED = 'API Key Whitelist Entry Added'
    API_KEY_ACCESS_LIST_ENTRY_ADDED = 'API Key Access List Entry Added'
    API_KEY_CREATED = 'API Key Created'
    API_KEY_DELETED = 'API Key Deleted'
    TEMP_GLOBAL_API_KEY_CREATED = 'Temp Global API Key Created'
    API_KEY_WHITELIST_ENTRY_DELETED = 'API Key Whitelist Entry Deleted'
    API_KEY_ACCESS_LIST_ENTRY_DELETED = 'API Key Access List Entry Deleted'
    API_KEY_ROLES_CHANGED = 'API Key Roles Changed'
    API_KEY_DESCRIPTION_CHANGED = 'API Key Description Changed'
    API_KEY_ADDED_TO_GROUP = 'API Key Added To Group'
    API_KEY_REMOVED_FROM_GROUP = 'API Key Removed From Group'
    MONITORING_AGENT_LOGS = 'Monitoring Agent Logs'
    MONITORING_AGENT_LOGS_CSV_DOWNLOAD = 'Monitoring Agent Logs CSV Download'
    AUTOMATION_AGENT_LOGS = 'Automation Agent Logs'
    AUTOMATION_AGENT_LOGS_CSV_DOWNLOAD = 'Automation Agent Logs CSV Download'
    BACKUP_AGENT_LOGS = 'Backup Agent Logs'
    BACKUP_AGENT_LOGS_CSV_DOWNLOAD = 'Backup Agent Logs CSV Download'
    MONITORING_MONGOD_LOGS = 'Monitoring Mongod Logs'
    VIEWED_MONITORING_HOST_SPECIFIC_MONGOD_LOGS = 'Viewed Monitoring Host Specific Mongod Logs'
    DOWNLOADED_MONITORING_HOST_SPECIFIC_MONGOD_LOGS = 'Downloaded Monitoring Host Specific Mongod Logs'
    DOWNLOADED_MONITORING_GROUP_MONGOD_LOGS = 'Downloaded Monitoring Group Mongod Logs'
    VIEWED_AUTOMATION_MONGOD_LOGS = 'Viewed Automation Mongod Logs'
    BACKUP_MONGOD_LOGS = 'Backup Mongod Logs'
    ATLAS_PROXY_LOGS = 'Atlas Proxy Logs'
    ATLAS_MONGOD_LOGS = 'Atlas Mongod Logs'
    ATLAS_MONGOS_LOGS = 'Atlas Mongos Logs'
    ATLAS_MONGOSQLD_LOGS = 'Atlas Mongosqld Logs'
    ATLAS_MONGOMIRROR_LOGS = 'Atlas Mongomirror Logs'
    ATLAS_MONGODUMP_LOGS = 'Atlas Mongodump Logs'
    ATLAS_MONGORESTORE_LOGS = 'Atlas Mongorestore Logs'
    ATLAS_MLAB_SHARED_MIGRATION_LOGS = 'Atlas Mlab Shared Migration Logs'
    ATLAS_MONGOT_LOGS = 'Atlas Mongot Logs'
    ATLAS_TENANT_UPGRADE_SNAPSHOTS = 'Atlas Tenant Upgrade Snapshots'
    VISUAL_PROFILER = 'Visual Profiler'
    REAL_TIME_PERFORMANCE_PANEL = 'Real Time Performance Panel'
    PERFORMANCE_ADVISOR = 'Performance Advisor'
    MONGODB_ACCESS_HISTORY = 'Mongodb Access History'
    TOGGLEABLE_FEATURE_FLAG = 'Toggleable Feature Flag'
    PUBLIC_API_MANAGED_SLOW_MS_FEATURE_FLAG = 'Public API Managed Slow Ms Feature Flag'
    MONITORING_DAILY_PING = 'Monitoring Daily Ping'
    MONITORING_LATEST_HOST_SPECIFIC_PING = 'Monitoring Latest Host Specific Ping'
    MONITORING_GROUP_PING = 'Monitoring Group Ping'
    PUBLIC_API_LATEST_MONITORING_GROUP_PING = 'Public API Latest Monitoring Group Ping'
    PUBLIC_API_LATEST_MONITORING_HOST_SPECIFIC_PING = 'Public API Latest Monitoring Host Specific Ping'
    MEMBER_ADDED = 'Member Added'
    MEMBER_REMOVED = 'Member Removed'
    CONFIGURATION_CHANGED = 'Configuration Changed'
    ENOUGH_HEALTHY_MEMBERS = 'Enough Healthy Members'
    TOO_FEW_HEALTHY_MEMBERS = 'Too Few Healthy Members'
    TOO_MANY_UNHEALTHY_MEMBERS = 'Too Many Unhealthy Members'
    TOO_MANY_ELECTIONS = 'Too Many Elections'
    REPLICATION_OPLOG_WINDOW_RUNNING_OUT = 'Replication Oplog Window Running Out'
    REPLICATION_OPLOG_WINDOW_HEALTHY = 'Replication Oplog Window Healthy'
    REPLICATION_OPLOG_WINDOW_TREND_HEALTHY = 'Replication Oplog Window Trend Healthy'
    PRIMARY_ELECTED = 'Primary Elected'
    ONE_PRIMARY = 'One Primary'
    MULTIPLE_PRIMARIES = 'Multiple Primaries'
    NO_PRIMARY = 'No Primary'
    CASE_CREATED = 'Case Created'
    AWS_AVAILABILITY_ZONE_OK = 'AWS Availability Zone Ok'
    AWS_AVAILABILITY_ZONE_DOWN = 'AWS Availability Zone Down'
    GROUP_ALERT_PROCESSING_ENABLED = 'Group Alert Processing Enabled'
    GROUP_ALERT_PROCESSING_DISABLED = 'Group Alert Processing Disabled'
    AZURE_REGION_OK = 'Azure Region Ok'
    AZURE_REGION_DOWN = 'Azure Region Down'
    SUFFICIENT_HEAD_FREE_SPACE = 'Sufficient Head Free Space'
    LOW_HEAD_FREE_SPACE = 'Low Head Free Space'
    LOW_HEAD_FREE_SPACE_PERCENT = 'Low Head Free Space Percent'
    DAEMON_UP = 'Daemon Up'
    DAEMON_DOWN = 'Daemon Down'
    OPLOG_TTL_RESIZE = 'Oplog Ttl Resize'
    THEFT_FAILED = 'Theft Failed'
    GROUP_CLOSED = 'Group Closed'
    GROUP_STUCK_IN_CLOSING = 'Group Stuck In Closing'
    BALANCER_OFF = 'Balancer Off'
    BALANCER_ON = 'Balancer On'
    INSIDE_SPACE_USED_THRESHOLD = 'Inside Space Used Threshold'
    OUTSIDE_SPACE_USED_THRESHOLD = 'Outside Space Used Threshold'
    SNAPSHOT_STORE_DELETED = 'Snapshot Store Deleted'
    SNAPSHOT_STORE_CONFIG_CHANGE = 'Snapshot Store Config Change'
    OPLOG_STORE_DELETED = 'Oplog Store Deleted'
    OPLOG_STORE_CONFIG_CHANGE = 'Oplog Store Config Change'
    SYNC_STORE_DELETED = 'Sync Store Deleted'
    SYNC_STORE_CONFIG_CHANGE = 'Sync Store Config Change'
    DAEMON_DELETED = 'Daemon Deleted'
    DAEMON_CONFIG_CHANGE = 'Daemon Config Change'
    GROUP_CONFIG_CHANGE = 'Group Config Change'
    PROVISIONING_CHANGE = 'Provisioning Change'
    JOB_CHANGE = 'Job Change'
    BREAK_JOB = 'Break Job'
    BULK_HEAD_MOVE = 'Bulk Head Move'
    SCHEDULE_GROOM = 'Schedule Groom'
    LOG_LEVEL_CHANGE = 'Log Level Change'
    CRON_JOB_COMPLETED = 'Cron Job Completed'
    CRON_JOB_FAILED = 'Cron Job Failed'
    CRON_JOB_ENABLED = 'Cron Job Enabled'
    CRON_JOB_DISABLED = 'Cron Job Disabled'
    BACKING_DATABASE_PROCESS_UP = 'Backing Database Process Up'
    BACKING_DATABASE_PROCESS_DOWN = 'Backing Database Process Down'
    BACKING_DATABASE_PROCESS_NO_STARTUP_WARNINGS = 'Backing Database Process No Startup Warnings'
    BACKING_DATABASE_PROCESS_STARTUP_WARNINGS = 'Backing Database Process Startup Warnings'
    GCP_ZONE_OK = 'GCP Zone Ok'
    GCP_ZONE_DOWN = 'GCP Zone Down'
    LOG_DEBUG_OVERRIDE_ACTIVE = 'Log Debug Override Active'
    MTM_CAPACITY_OK = 'MTM Capacity Ok'
    MTM_CAPACITY_LOW = 'MTM Capacity Low'
    TEAM_CREATED = 'Team Created'
    TEAM_DELETED = 'Team Deleted'
    TEAM_UPDATED = 'Team Updated'
    TEAM_NAME_CHANGED = 'Team Name Changed'
    TEAM_ADDED_TO_GROUP = 'Team Added To Group'
    TEAM_REMOVED_FROM_GROUP = 'Team Removed From Group'
    TEAM_ROLES_MODIFIED = 'Team Roles Modified'
    MULTI_FACTOR_AUTH_RESET_EMAIL_SENT_AUDIT = 'Multi Factor Auth Reset Email Sent Audit'
    MULTI_FACTOR_AUTH_RESET_AUDIT = 'Multi Factor Auth Reset Audit'
    MULTI_FACTOR_AUTH_UPDATED_AUDIT = 'Multi Factor Auth Updated Audit'
    JOINED_GROUP = 'Joined Group'
    JOINED_ORG = 'Joined Org'
    JOINED_TEAM = 'Joined Team'
    REMOVED_FROM_GROUP = 'Removed From Group'
    INVITED_TO_GROUP = 'Invited To Group'
    INVITED_TO_ORG = 'Invited To Org'
    REQUESTED_TO_JOIN_GROUP = 'Requested To Join Group'
    REMOVED_FROM_ORG = 'Removed From Org'
    REMOVED_FROM_TEAM = 'Removed From Team'
    PASSWORD_RESET_EMAIL_SENT_AUDIT = 'Password Reset Email Sent Audit'
    PASSWORD_RESET_FORM_VIEWED_AUDIT = 'Password Reset Form Viewed Audit'
    PASSWORD_RESET_FAILED_AUDIT = 'Password Reset Failed Audit'
    PASSWORD_RESET_AUDIT = 'Password Reset Audit'
    PASSWORD_UPDATED_AUDIT = 'Password Updated Audit'
    USER_VERIFICATION_EMAIL_SENT_AUDIT = 'User Verification Email Sent Audit'
    USER_EMAIL_VERIFIED_AUDIT = 'User Email Verified Audit'
    USER_EMAIL_ADDRESS_CHANGED_AUDIT = 'User Email Address Changed Audit'
    USER_SALESFORCE_CONTACT_ID_CHANGED_AUDIT = 'User Salesforce Contact Id Changed Audit'
    USER_FIRST_NAME_LAST_NAME_CHANGED_AUDIT = 'User First Name Last Name Changed Audit'
    USER_ROLES_CHANGED_AUDIT = 'User Roles Changed Audit'
    SUCCESSFUL_LOGIN_AUDIT = 'Successful Login Audit'
    UNSUCCESSFUL_LOGIN_AUDIT = 'Unsuccessful Login Audit'
    ACCOUNT_LOCKED_AUDIT = 'Account Locked Audit'
    ACCOUNT_UNLOCKED_AUDIT = 'Account Unlocked Audit'
    USER_CREATED_AUDIT = 'User Created Audit'
    JOIN_GROUP_REQUEST_DENIED_AUDIT = 'Join Group Request Denied Audit'
    JOIN_GROUP_REQUEST_APPROVED_AUDIT = 'Join Group Request Approved Audit'
    USER_DELETED_AUDIT = 'User Deleted Audit'
    USER_HARD_DELETED_AUDIT = 'User Hard Deleted Audit'
    USER_RESTORED_AUDIT = 'User Restored Audit'
    USER_NAME_OFAC_HIT = 'User Name OFAC Hit'
    USER_UNEMBARGOED = 'User Unembargoed'
    ORG_FLEX_CONSULTING_PURCHASED = 'Org Flex Consulting Purchased'
    ORG_FLEX_CONSULTING_PURCHASE_FAILED = 'Org Flex Consulting Purchase Failed'
    USER_PROFILE_CHANGED_AUDIT = 'User Profile Changed Audit'
    MONGODB_USER_X509_CERT_CREATED = 'Mongodb User X509 Cert Created'
    MONGODB_USER_X509_CERT_REVOKED = 'Mongodb User X509 Cert Revoked'
    CUSTOMER_X509_CRL_UPDATED = 'Customer X509 CRL Updated'
    CONTAINER_SUBNETS_UPDATE_REQUESTED = 'Container Subnets Update Requested'
    CLEAR_UNPROVISIONED_TARGET_GROUPS_REQUESTED = 'Clear Unprovisioned Target Groups Requested'
    GLOBAL_WHITELIST_ENTRY_CREATED = 'Global Whitelist Entry Created'
    GLOBAL_WHITELIST_ENTRY_UPDATED = 'Global Whitelist Entry Updated'
    GLOBAL_WHITELIST_ENTRY_DELETED = 'Global Whitelist Entry Deleted'
    GLOBAL_ACCESS_LIST_ENTRY_CREATED = 'Global Access List Entry Created'
    GLOBAL_ACCESS_LIST_ENTRY_UPDATED = 'Global Access List Entry Updated'
    GLOBAL_ACCESS_LIST_ENTRY_DELETED = 'Global Access List Entry Deleted'
    NDS_X509_USER_AUTHENTICATION_MANAGED_USER_CERTS_EXPIRATION_RESOLVED = 'Nds X509 User Authentication Managed User Certs Expiration Resolved'
    NDS_X509_USER_AUTHENTICATION_MANAGED_USER_CERTS_EXPIRATION_CHECK = 'Nds X509 User Authentication Managed User Certs Expiration Check'
    NDS_X509_USER_AUTHENTICATION_CUSTOMER_CA_EXPIRATION_RESOLVED = 'Nds X509 User Authentication Customer Ca Expiration Resolved'
    NDS_X509_USER_AUTHENTICATION_CUSTOMER_CA_EXPIRATION_CHECK = 'Nds X509 User Authentication Customer Ca Expiration Check'
    NDS_X509_USER_AUTHENTICATION_CUSTOMER_CRL_EXPIRATION_RESOLVED = 'Nds X509 User Authentication Customer CRL Expiration Resolved'
    NDS_X509_USER_AUTHENTICATION_CUSTOMER_CRL_EXPIRATION_CHECK = 'Nds X509 User Authentication Customer CRL Expiration Check'
    ONLINE_ARCHIVE_INSUFFICIENT_INDEXES_CHECK = 'Online Archive Insufficient Indexes Check'
    ONLINE_ARCHIVE_INSUFFICIENT_INDEXES_RESOLVED = 'Online Archive Insufficient Indexes Resolved'
    ONLINE_ARCHIVE_MAX_CONSECUTIVE_OFFLOADS_CHECK = 'Online Archive Max Consecutive Offloads Check'
    ONLINE_ARCHIVE_UP_TO_DATE = 'Online Archive Up To Date'
    ONLINE_ARCHIVE_CREATED = 'Online Archive Created'
    ONLINE_ARCHIVE_DELETED = 'Online Archive Deleted'
    ONLINE_ARCHIVE_UPDATED = 'Online Archive Updated'
    ONLINE_ARCHIVE_PAUSE_REQUESTED = 'Online Archive Pause Requested'
    ONLINE_ARCHIVE_PAUSED = 'Online Archive Paused'
    ONLINE_ARCHIVE_ACTIVE = 'Online Archive Active'
    ONLINE_ARCHIVE_ORPHANED = 'Online Archive Orphaned'
    ONLINE_ARCHIVE_AGENT_LOGS = 'Online Archive Agent Logs'
    ONLINE_ARCHIVE_AGENT_LOGS_CSV_DOWNLOAD = 'Online Archive Agent Logs CSV Download'
    ENDPOINT_SERVICE_CREATED = 'Endpoint Service Created'
    ENDPOINT_SERVICE_DELETED = 'Endpoint Service Deleted'
    INTERFACE_ENDPOINT_CREATED = 'Interface Endpoint Created'
    INTERFACE_ENDPOINT_DELETED = 'Interface Endpoint Deleted'
    INTERFACE_ENDPOINT_PATCHED = 'Interface Endpoint Patched'
    USAGE_DATA_ENABLED = 'Usage Data Enabled'
    USAGE_DATA_DISABLED = 'Usage Data Disabled'
    USAGE_DATA_RAW_DOWNLOAD = 'Usage Data Raw Download'
    MAINTENANCE_IN_ADVANCED = 'Maintenance In Advanced'
    MAINTENANCE_STARTED = 'Maintenance Started'
    MAINTENANCE_NO_LONGER_NEEDED = 'Maintenance No Longer Needed'
    MAINTENANCE_AUTO_DEFERRED = 'Maintenance Auto Deferred'
    ORG_POLICY_CREATED = 'Org Policy Created'
    ORG_POLICY_DELETED = 'Org Policy Deleted'
    ORG_POLICY_EDITED = 'Org Policy Edited'
    ORG_POLICY_CLONED = 'Org Policy Cloned'
    CPS_SNAPSHOT_PROCESSING = 'Cps Snapshot Processing'
    CPS_SNAPSHOT_IN_QUEUE = 'Cps Snapshot In Queue'
    CPS_SNAPSHOT_SUCCESSFUL = 'Cps Snapshot Successful'
    CPS_SNAPSHOT_FALLBACK_SUCCESSFUL = 'Cps Snapshot Fallback Successful'
    CPS_SNAPSHOT_IN_TIME = 'Cps Snapshot In Time'
    CPS_SNAPSHOT_BEHIND = 'Cps Snapshot Behind'
    CPS_SNAPSHOT_FALLBACK_FAILED = 'Cps Snapshot Fallback Failed'
    CPS_OPLOG_BEHIND = 'Cps Oplog Behind'
    CPS_OPLOG_CAUGHT_UP = 'Cps Oplog Caught Up'
    CPS_OPLOG_NOT_CONTIGUOUS = 'Cps Oplog Not Contiguous'
    CPS_RESTORE_SUCCESSFUL = 'Cps Restore Successful'
    CPS_EXPORT_SUCCESSFUL = 'Cps Export Successful'
    CPS_RESTORE_FAILED = 'Cps Restore Failed'
    CPS_EXPORT_FAILED = 'Cps Export Failed'
    CPS_SNAPSHOT_DOWNLOAD_REQUEST_FAILED = 'Cps Snapshot Download Request Failed'
    FTS_INDEX_BUILD_COMPLETE = 'Fts Index Build Complete'
    FTS_INDEX_BUILD_FAILED = 'Fts Index Build Failed'
    HOST_USAGE_TYPE_ASSIGNMENT_CHANGE = 'Host Usage Type Assignment Change'
    PHYSICAL_HOST_USAGE_TYPE_ASSIGNMENT_CHANGE = 'Physical Host Usage Type Assignment Change'
    FEDERATION_SETTINGS_CREATED = 'Federation Settings Created'
    FEDERATION_SETTINGS_DELETED = 'Federation Settings Deleted'
    FEDERATION_SETTINGS_UPDATED = 'Federation Settings Updated'
    IDENTITY_PROVIDER_CREATED = 'Identity Provider Created'
    IDENTITY_PROVIDER_UPDATED = 'Identity Provider Updated'
    IDENTITY_PROVIDER_DELETED = 'Identity Provider Deleted'
    IDENTITY_PROVIDER_ACTIVATED = 'Identity Provider Activated'
    DOMAINS_ASSOCIATED = 'Domains Associated'
    DOMAIN_CREATED = 'Domain Created'
    DOMAIN_DELETED = 'Domain Deleted'
    DOMAIN_VERIFIED = 'Domain Verified'
    ORG_SETTINGS_CONFIGURED = 'Org Settings Configured'
    ORG_SETTINGS_UPDATED = 'Org Settings Updated'
    ORG_SETTINGS_DELETED = 'Org Settings Deleted'
    RESTRICT_ORG_MEMBERSHIP_ENABLED = 'Restrict Org Membership Enabled'
    RESTRICT_ORG_MEMBERSHIP_DISABLED = 'Restrict Org Membership Disabled'
    ROLE_MAPPING_CREATED = 'Role Mapping Created'
    ROLE_MAPPING_UPDATED = 'Role Mapping Updated'
    ROLE_MAPPING_DELETED = 'Role Mapping Deleted'
    INTEGRATION_CONFIGURED = 'Integration Configured'
    INTEGRATION_REMOVED = 'Integration Removed'
    MONGOT_BEHIND = 'Mongot Behind'
    MONOGT_UP_TO_DATE = 'Monogt Up To Date'
    MONGOT_CRASH_LOG_CREATED = 'Mongot Crash Log Created'
    MONGOT_CRASH_LOG_RESOLVED = 'Mongot Crash Log Resolved'
    CRON_JOB_MANUALLY_DISABLED_AUDIT = 'Cron Job Manually Disabled Audit'
    CRON_JOB_MANUALLY_ENABLED_AUDIT = 'Cron Job Manually Enabled Audit'
    CRON_JOB_MANUALLY_TRIGGERED_AUDIT = 'Cron Job Manually Triggered Audit'
    LONG_RUNNING_MOVE = 'Long Running Move'
    PENDING_INDEXES_DELETED = 'Pending Indexes Deleted'
    PROCESS_RESTART_REQUESTED = 'Process Restart Requested'
    AUTO_INDEXING_ENABLED = 'Auto Indexing Enabled'
    AUTO_INDEXING_DISABLED = 'Auto Indexing Disabled'
    AUTO_INDEXING_INDEX_BUILD_SUBMITTED = 'Auto Indexing Index Build Submitted'
    AUTO_INDEXING_SLOW_INDEX_BUILD = 'Auto Indexing Slow Index Build'
    AUTO_INDEXING_STALLED_INDEX_BUILD = 'Auto Indexing Stalled Index Build'
    AUTO_INDEXING_FAILED_INDEX_BUILD = 'Auto Indexing Failed Index Build'
    AUTO_INDEXING_COMPLETED_INDEX_BUILD = 'Auto Indexing Completed Index Build'
    SERVERLESS_MIGRATION_INITIATED = 'Serverless Migration Initiated'
    SERVERLESS_MIGRATION_FAILED = 'Serverless Migration Failed'
    SERVERLESS_MIGRATION_CLEANUP_FAILED = 'Serverless Migration Cleanup Failed'
    SERVERLESS_MIGRATION_COMPLETED = 'Serverless Migration Completed'
    SERVERLESS_AUTO_SCALING_TRIGGERED = 'Serverless Auto Scaling Triggered'
    SERVERLESS_AUTO_SCALING_INITIATED = 'Serverless Auto Scaling Initiated'
    SERVERLESS_AUTO_SCALING_SKIPPED = 'Serverless Auto Scaling Skipped'
    SERVERLESS_VERTICAL_SCALING_INITIATED = 'Serverless Vertical Scaling Initiated'
    SERVERLESS_HORIZONTAL_SCALING_INITIATED = 'Serverless Horizontal Scaling Initiated'
    USER_CANCELED_EMAIL_CHANGE_AUDIT = 'User Canceled Email Change Audit'
    USER_CANCELED_EMAIL_VALIDATION_AUDIT = 'User Canceled Email Validation Audit'
    DATA_LAKE_QUERY_LOGS_DOWNLOADED = 'Data Lake Query Logs Downloaded'
    USER_REQUESTED_EMAIL_ADDRESS_CHANGE_AUDIT = 'User Requested Email Address Change Audit'
    ONLINE_ARCHIVE_QUERY_LOGS_DOWNLOADED = 'Online Archive Query Logs Downloaded'
    HOST_MONGOT_CRASHING_OOM = 'Host Mongot Crashing Oom'
    HOST_MONGOT_RECOVERED_OOM = 'Host Mongot Recovered Oom'
    SELF_SERVE_DELETION_REQUESTED_AUDIT = 'Self Serve Deletion Requested Audit'
    NDS_BACKGROUND_PROCESSING_DISABLED = 'Nds Background Processing Disabled'
    NDS_BACKGROUND_PROCESSING_ENABLED = 'Nds Background Processing Enabled'
    METER_USAGE_UNDELETED_THROUGH_API = 'Meter Usage Undeleted Through API'
    SYNC_FAILURE = 'Sync Failure'
    TRIGGER_FAILURE = 'Trigger Failure'
    URL_CONFIRMATION = 'Url Confirmation'
    SUCCESSFUL_DEPLOY = 'Successful Deploy'
    DEPLOYMENT_FAILURE = 'Deployment Failure'
    REQUEST_RATE_LIMIT = 'Request Rate Limit'
    ORG_IDP_CERTIFICATE_ABOUT_TO_EXPIRE = 'Org Idp Certificate About To Expire'
    ORG_IDP_CERTIFICATE_CURRENT = 'Org Idp Certificate Current'
    SERVERLESS_DEPLOYMENT_CREATED = 'Serverless Deployment Created'
    SERVERLESS_DEPLOYMENT_DELETED = 'Serverless Deployment Deleted'
    CLUSTER_INSTANCE_ADMIN_BACKUP_SNAPSHOT_REQUESTED = 'Cluster Instance Admin Backup Snapshot Requested'
    ATLAS_ENVOY_SERVERLESS_LOGS = 'Atlas Envoy Serverless Logs'
    ORG_LIMIT_UPDATED = 'Org Limit Updated'
    ATLAS_DATA_LAKE_LOGS = 'Atlas Data Lake Logs'
    PIT_LOGS = 'Pit Logs'
    CPS_BACKUP_LOGS = 'Cps Backup Logs'
    INSIDE_SERVERLESS_METRIC_THRESHOLD = 'Inside Serverless Metric Threshold'
    OUTSIDE_SERVERLESS_METRIC_THRESHOLD = 'Outside Serverless Metric Threshold'
    CLOUD_PROVIDER_THROTTLE_STATE_MODIFIED = 'Cloud Provider Throttle State Modified'
    PROJECT_THROTTLE_STATE_MODIFIED = 'Project Throttle State Modified'
    CLUSTER_THROTTLE_STATE_MODIFIED = 'Cluster Throttle State Modified'
    FORCE_MIGRATE_FROM_AVAILABILITY_SETS = 'Force Migrate From Availability Sets'
    TENANT_CLUSTER_LIMIT_MODIFIED = 'Tenant Cluster Limit Modified'
    SERVERLESS_DEPLOYMENT_UPDATED = 'Serverless Deployment Updated'
    INITIATE_SALESFORCE_SERVICE_CLOUD_SYNC = 'Initiate Salesforce Service Cloud Sync'
    FTS_INDEX_CREATED = 'Fts Index Created'
    FTS_INDEX_UPDATED = 'Fts Index Updated'
    FTS_INDEX_DELETED = 'Fts Index Deleted'
    INITIATE_SFDC_SYNC = 'Initiate Sfdc Sync'
    SERVERLESS_DEPLOYMENT_INSTANCE_REPLACED = 'Serverless Deployment Instance Replaced'
    AWS_BILLING_ACCOUNT_CREDIT_ISSUED = 'AWS Billing Account Credit Issued'
    AWS_USAGE_REPORTED = 'AWS Usage Reported'
    GCP_USAGE_REPORTED = 'GCP Usage Reported'
    SERVERLESS_DEPLOYMENT_INSTANCE_REBOOTED = 'Serverless Deployment Instance Rebooted'
    HOST_MONGOT_CRASHING_OOM_MTM = 'Host Mongot Crashing Oom MTM'
    HOST_MONGOT_RECOVERED_OOM_MTM = 'Host Mongot Recovered Oom MTM'
    SUPPORT_EMAILS_SENT_SUCCESSFULLY = 'Support Emails Sent Successfully'
    PRIVATE_NETWORK_ENDPOINT_ENTRY_ADDED = 'Private Network Endpoint Entry Added'
    PRIVATE_NETWORK_ENDPOINT_ENTRY_REMOVED = 'Private Network Endpoint Entry Removed'
    PRIVATE_NETWORK_ENDPOINT_ENTRY_UPDATED = 'Private Network Endpoint Entry Updated'
    CLUSTER_MONGOT_PROCESS_ARGS_UPDATE_SUBMITTED = 'Cluster Mongot Process Args Update Submitted'
    CLUSTER_SERVER_PARAMETERS_UPDATE_SUBMITTED = 'Cluster Server Parameters Update Submitted'
    M0_CLUSTERS_PER_GROUP_WITHIN_LIMIT = 'M0 Clusters Per Group Within Limit'
    MAX_M0_CLUSTERS_PER_GROUP_EXCEEDED = 'Max M0 Clusters Per Group Exceeded'
    CLUSTER_CONNECTION_CREATE_COLLECTION = 'Cluster Connection Create Collection'
    AUTO_HEALING_ACTION = 'Auto Healing Action'
    SUPPORT_EMAILS_SENT_FAILURE = 'Support Emails Sent Failure'
    EXTRA_MAINTENANCE_DEFERRAL_GRANTED = 'Extra Maintenance Deferral Granted'
    ADMIN_NOTE_UPDATED = 'Admin Note Updated'
    ONLINE_ARCHIVE_RESUMED = 'Online Archive Resumed'
    ONLINE_ARCHIVE_PAUSED_DUE_TO_FRS_JOB_ERRORS_CHECK = 'Online Archive Paused Due To Frs Job Errors Check'
    ORG_ADMIN_LOCKED = 'Org Admin Locked'
    REVOKED_EMPLOYEE_ACCESS_BYPASS = 'Revoked Employee Access Bypass'
    HOST_ENOUGH_DISK_SPACE = 'Host Enough Disk Space'
    HOST_NOT_ENOUGH_DISK_SPACE = 'Host Not Enough Disk Space'
    AWS_SELF_SERVE_ACCOUNT_LINKED = 'AWS Self Serve Account Linked'
    AWS_SELF_SERVE_ACCOUNT_LINK_PENDING = 'AWS Self Serve Account Link Pending'
    AWS_SELF_SERVE_ACCOUNT_CANCELLED = 'AWS Self Serve Account Cancelled'
    AWS_SELF_SERVE_ACCOUNT_LINK_FAILED = 'AWS Self Serve Account Link Failed'
    OS_MAINTENANCE_RESTART = 'Os Maintenance Restart'
    DEVICE_CODE_GENERATED = 'Device Code Generated'
    SUCCESSFUL_DEVICE_CONFIRMATION_AUDIT = 'Successful Device Confirmation Audit'
    GROUP_AUTOMATION_CONFIG_PUBLISHED = 'Group Automation Config Published'
    CLUSTER_AUTOMATION_CONFIG_PUBLISHED = 'Cluster Automation Config Published'
    ORG_CONNECTED_TO_VERCEL = 'Org Connected To Vercel'
    DEVICE_CODE_CONFIRMED = 'Device Code Confirmed'
    DEVICE_CODE_AUTHORIZED = 'Device Code Authorized'
    UNLINKED_ORG = 'Unlinked Org'
    ORG_UNLINKED_FROM_PAYING_ORG = 'Org Unlinked From Paying Org'
    CLUSTER_LINKED_TO_VERCEL = 'Cluster Linked To Vercel'
    DEVICE_REVOKED = 'Device Revoked'
    ROLLING_INDEX_FAILED_INDEX_BUILD = 'Rolling Index Failed Index Build'
    SCHEDULE_CREATED = 'Schedule Created'
    SCHEDULE_DELETE_REQUESTED = 'Schedule Delete Requested'
    SCHEDULE_PAUSE_REQUESTED = 'Schedule Pause Requested'
    SCHEDULE_ENABLE_REQUESTED = 'Schedule Enable Requested'
    SCHEDULE_UPDATE_REQUESTED = 'Schedule Update Requested'
    SCHEDULE_MANUAL_TRIGGER_REQUESTED = 'Schedule Manual Trigger Requested'
    SCHEDULE_RESUMED = 'Schedule Resumed'
    SCHEDULE_SIDELINED = 'Schedule Sidelined'
    DEVICE_REFRESHED = 'Device Refreshed'
    TENANT_ENDPOINT_SERVICE_DEPLOYMENT_CREATED = 'Tenant Endpoint Service Deployment Created'
    TENANT_ENDPOINT_SERVICE_CREATED = 'Tenant Endpoint Service Created'
    TENANT_ENDPOINT_SERVICE_AVAILABLE = 'Tenant Endpoint Service Available'
    TENANT_ENDPOINT_SERVICE_DEPLOYMENT_DELETE_REQUESTED = 'Tenant Endpoint Service Deployment Delete Requested'
    TENANT_ENDPOINT_SERVICE_DELETED = 'Tenant Endpoint Service Deleted'
    TENANT_ENDPOINT_SERVICE_DEPLOYMENT_DELETED = 'Tenant Endpoint Service Deployment Deleted'
    TENANT_ENDPOINT_SERVICE_DEPLOYMENT_NUM_DESIRED_ENDPOINT_SERVICES_INCREASED = 'Tenant Endpoint Service Deployment Num Desired Endpoint Services Increased'
    SERVERLESS_DEPLOYMENT_ENDPOINT_SERVICE_LINKED = 'Serverless Deployment Endpoint Service Linked'
    SERVERLESS_DEPLOYMENT_ENDPOINT_SERVICE_UNLINKED = 'Serverless Deployment Endpoint Service Unlinked'
    CLUSTER_CONNECTION_SAMPLE_COLLECTION_FIELD_NAMES = 'Cluster Connection Sample Collection Field Names'
    ROLLING_INDEX_SUCCESS_INDEX_BUILD = 'Rolling Index Success Index Build'
    ORG_CONNECTION_UNINSTALLED_FROM_VERCEL = 'Org Connection Uninstalled From Vercel'
    SIGNING_KEY_ROTATION_TRIGGERED = 'Signing Key Rotation Triggered'
    SIGNING_KEY_ROTATED = 'Signing Key Rotated'
    GCP_SELF_SERVE_ACCOUNT_LINK_PENDING = 'GCP Self Serve Account Link Pending'
    GCP_SELF_SERVE_ACCOUNT_LINK_FAILED = 'GCP Self Serve Account Link Failed'
    QUOTA_USAGE_EXCEEDED_EVENT = 'Quota Usage Exceeded Event'
    QUOTA_USAGE_WARNING_EVENT = 'Quota Usage Warning Event'
    QUOTA_USAGE_CRITICAL_WARNING_EVENT = 'Quota Usage Critical Warning Event'
    QUOTA_USAGE_CRITICAL_WARNING_PAGE_EVENT = 'Quota Usage Critical Warning Page Event'
    PROJECT_OPERATIONAL_LIMIT_UPDATED = 'Project Operational Limit Updated'
    GCP_SELF_SERVE_ACCOUNT_LINKED = 'GCP Self Serve Account Linked'
    TENANT_ENDPOINT_CREATED = 'Tenant Endpoint Created'
    TENANT_ENDPOINT_RESERVED = 'Tenant Endpoint Reserved'
    TENANT_ENDPOINT_RESERVATION_FAILED = 'Tenant Endpoint Reservation Failed'
    TENANT_ENDPOINT_UPDATED = 'Tenant Endpoint Updated'
    TENANT_ENDPOINT_AVAILABLE = 'Tenant Endpoint Available'
    TENANT_ENDPOINT_FAILED = 'Tenant Endpoint Failed'
    TENANT_ENDPOINT_DELETING = 'Tenant Endpoint Deleting'
    TENANT_ENDPOINT_DELETED = 'Tenant Endpoint Deleted'
    ORG_DISCONNECTED_TO_VERCEL = 'Org Disconnected To Vercel'
    ONLINE_ARCHIVE_DATA_EXPIRATION_RULE_ENABLED = 'Online Archive Data Expiration Rule Enabled'
    ONLINE_ARCHIVE_DATA_EXPIRATION_RULE_UPDATED = 'Online Archive Data Expiration Rule Updated'
    ONLINE_ARCHIVE_DATA_EXPIRATION_RULE_DISABLED = 'Online Archive Data Expiration Rule Disabled'
    GCP_SELF_SERVE_ACCOUNT_CANCELLED = 'GCP Self Serve Account Cancelled'
    INGESTION_PIPELINE_DELETED = 'Ingestion Pipeline Deleted'
    OS_TUNE_FILE_OVERRIDES = 'Os Tune File Overrides'
    BILLING_EMAIL_ADDRESS_ADDED = 'Billing Email Address Added'
    BILLING_EMAIL_ADDRESS_CHANGED = 'Billing Email Address Changed'
    BILLING_EMAIL_ADDRESS_REMOVED = 'Billing Email Address Removed'
    INDEX_FAILED_INDEX_BUILD = 'Index Failed Index Build'
    INDEX_SUCCESS_INDEX_BUILD = 'Index Success Index Build'
    INGESTION_PIPELINE_CREATED = 'Ingestion Pipeline Created'
    INGESTION_PIPELINE_UPDATED = 'Ingestion Pipeline Updated'
    CLUSTER_PREFERRED_CPU_ARCHITECTURE_MODIFIED = 'Cluster Preferred Cpu Architecture Modified'
    USER_ACCOUNT_EMAIL_ADDRESS_CHANGED_AUDIT = 'User Account Email Address Changed Audit'
    ONLINE_ARCHIVE_DATA_EXPIRATION_DID_NOT_RUN = 'Online Archive Data Expiration Did Not Run'
    ONLINE_ARCHIVE_DATA_EXPIRATION_RESOLVED = 'Online Archive Data Expiration Resolved'
    LOG_FORWARDER_FAILURE = 'Log Forwarder Failure'
    CLUSTER_UNLINKED_FROM_VERCEL = 'Cluster Unlinked From Vercel'



# Copyright (c) 2022 Matthew G. Monteleone
# 
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
# 
#     http://www.apache.org/licenses/LICENSE-2.0
# 
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""
Network module

Module which handles the basic network operations with the Atlas API>
"""

import requests
from requests.auth import HTTPDigestAuth, HTTPBasicAuth
from atlasapi.settings import Settings
from atlasapi.errors import *
import logging
from json import dumps
from io import BytesIO
from typing import Union

logger = logging.getLogger('network')
logger.setLevel(logging.WARNING)


def merge(dict1, dict2):
    return dict2.update(dict1)

class Network:
    """Network constructor
    
    Args:
        user (str): user
        password (str): password
    """

    def __init__(self, user, password, AuthMethod: Union[HTTPDigestAuth, HTTPBasicAuth] = HTTPDigestAuth):
        self.user: str = user
        self.password: str = password
        self.auth_method: Union[HTTPDigestAuth, HTTPBasicAuth] = AuthMethod

    def answer(self, c, details: Union[dict, BytesIO]):
        """Answer will provide all necessary feedback for the caller
        
        Args:
            c (int): HTTP Code
            details (dict): Response payload
        
        Returns:
            dict: Response payload
            
        Raises:
            ErrAtlasBadRequest
            ErrAtlasUnauthorized
            ErrAtlasForbidden
            ErrAtlasNotFound
            ErrAtlasMethodNotAllowed
            ErrAtlasConflict
            ErrAtlasServerErrors
        
        """
        if c in [Settings.SUCCESS, Settings.CREATED, Settings.ACCEPTED, Settings.NO_CONTENT]:
            return details
        elif c == Settings.BAD_REQUEST:
            raise ErrAtlasBadRequest(c, details)
        elif c == Settings.UNAUTHORIZED:
            raise ErrAtlasUnauthorized(c, details)
        elif c == Settings.FORBIDDEN:
            raise ErrAtlasForbidden(c, details)
        elif c == Settings.NOTFOUND:
            raise ErrAtlasNotFound(c, details)
        elif c == Settings.METHOD_NOT_ALLOWED:
            raise ErrAtlasMethodNotAllowed(c, details)
        elif c == Settings.CONFLICT:
            raise ErrAtlasConflict(c, details)
        else:
            # Settings.SERVER_ERRORS
            raise ErrAtlasServerErrors(c, details)

    def get_file(self, uri):
        """Get request which returns a binary file

        Args:
            uri (str): URI

        Returns:
            Binary File: API response as file

        Raises:
            Exception: Network issue
        """
        r = None

        try:
            file_obj = BytesIO()
            r = requests.get(uri,
                             allow_redirects=False,
                             stream=True,
                             timeout=Settings.file_request_timeout,
                             headers={},
                             auth=self.auth_method(self.user, self.password))
            logger.debug("Auth information = {} {}".format(self.user, self.password))

            for chunk in r.iter_content(chunk_size=1024):
                # writing one chunk at a time to  file
                if chunk:
                    logger.warning("Writing 1 Kbyte chunk to the file like object")
                    file_obj.write(chunk)
            logger.info("---- Completed downloading the file. ----")
            return self.answer(r.status_code, file_obj)

        except Exception:
            logger.warning('Request: {}'.format(r.request.__dict__))
            logger.warning('Response: {}'.format(r.__dict__))
            raise
        finally:
            if r:
                r.connection.close()

    def get(self, uri):
        """Get request
        
        Args:
            uri (str): URI
            
        Returns:
            Json: API response
            
        Raises:
            Exception: Network issue
        """
        r = None

        try:
            r = requests.get(uri,
                             allow_redirects=True,
                             timeout=Settings.requests_timeout,
                             headers={},
                             auth=self.auth_method(self.user, self.password))
            logger.debug("Auth information = {} {}".format(self.user, self.password))

            return self.answer(r.status_code, r.json())
        except Exception as e:
            logger.warning('Request: {}'.format(r.request.__dict__))
            logger.warning('Response: {}'.format(r.__dict__))
            raise e
        finally:
            if r:
                r.connection.close()

    def get_big(self, uri, params: dict = None):
        """Get request (max results)

        This is a temporary fix until we re-factor pagination.

        Args:
            uri (str): URI

        Returns:
            Json: API response

        Raises:
            Exception: Network issue
        """
        r = None
        print(f"Revieved the following parameters {params}")
        if params:
            logger.warning(f"Revieved the following parameters {params}")
            merge({'itemsPerPage': Settings.itemsPerPage},params)
            logger.warning(f"The parameters are now {params}")
        else:
            params = {'itemsPerPage': Settings.itemsPerPage}

        try:
            logger.warning(f"The parameters object is {params}")
            r = requests.get(uri,
                             allow_redirects=True,
                             params=params,
                             timeout=Settings.requests_timeout,
                             headers={},
                             auth=self.auth_method(self.user, self.password))
            logger.debug("Auth information = {} {}".format(self.user, self.password))

            return self.answer(r.status_code, r.json())
        except Exception as e:
            raise e
        finally:
            if r:
                r.connection.close()

    def post(self, uri, payload):
        """Post request
        
        Args:
            uri (str): URI
            payload (dict): Content to post 
            
        Returns:
            Json: API response
            
        Raises:
            Exception: Network issue
        """
        r = None

        try:
            r = requests.post(uri,
                              json=payload,
                              allow_redirects=True,
                              timeout=Settings.requests_timeout,
                              headers={"Content-Type": "application/json"},
                              auth=self.auth_method(self.user, self.password))
            return self.answer(r.status_code, r.json())
        except:
            raise
        finally:
            if r:
                r.connection.close()

    def patch(self, uri, payload):
        """Patch request
        
        Args:
            uri (str): URI
            payload (dict): Content to patch
            
        Returns:
            Json: API response
            
        Raises:
            Exception: Network issue
        """
        r = None
        try:
            r = requests.patch(uri,
                               json=payload,
                               allow_redirects=True,
                               timeout=Settings.requests_timeout,
                               headers={"Content-Type": "application/json"},
                               auth=self.auth_method(self.user, self.password))

            try:
                output = r.json()
            except:
                logger.warning("PATCH doesnt return data!")
                output = {}

            return self.answer(r.status_code, output)
        except Exception as e:

            raise e
        finally:
            if r:
                r.connection.close()

    def delete(self, uri):
        """Delete request
        
        Args:
            uri (str): URI
            
        Returns:
            Json: API response
            
        Raises:
            Exception: Network issue
        """
        r = None

        try:
            r = requests.delete(uri,
                                allow_redirects=True,
                                timeout=Settings.requests_timeout,
                                headers={},
                                auth=self.auth_method(self.user, self.password))
            return self.answer(r.status_code, {"deleted": True})
        except Exception as e:
            raise e
        finally:
            if r:
                r.connection.close()



# Copyright (c) 2019 Matthew G. Monteleone
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""
Maint Window Module

The maintenanceWindow resource provides access to retrieve or update the current Atlas project maintenance window.
To learn more about Maintenance Windows, see the Set Preferred Cluster Maintenance Start Time
setting on the View/Modify Project Settings page.

"""
from enum import Enum
from pprint import pprint
import logging
from json import dumps
from enum import IntEnum

logger = logging.getLogger('maintenance_window')


class Weekdays(Enum):
    SUNDAY = 1
    MONDAY = 2
    TUESDAY = 3
    WEDNESDAY = 4
    THURSDAY = 5
    FRIDAY = 6
    SATURDAY = 7


class MaintenanceWindow(object):
    def __init__(self, day_of_week: Weekdays = Weekdays.SUNDAY, hour_of_day: int = 23, number_of_deferrals: int = 1,
                 start_asap: bool = False):
        """
        Stores the definition of maint window configuration for a group/project.

        Args:
            day_of_week: The day of week that maint should run
            hour_of_day: the hour of the day (24 based) to run maint
            number_of_deferrals:
            start_asap:
        """
        self.startASAP = start_asap
        self.numberOfDeferrals = number_of_deferrals
        self.hourOfDay = hour_of_day
        self.dayOfWeek = day_of_week

    @classmethod
    def from_dict(cls, data_dict: dict):
        """
        Creates a maint window definition from a dict.
        Args:
            data_dict: An atlas formated dict

        Returns:

        """
        day_of_week: Weekdays = Weekdays(data_dict.get('dayOfWeek', None))
        hour_of_day = data_dict.get('hourOfDay', None)
        number_of_deferrals = data_dict.get('numberOfDeferrals', None)
        start_asap = data_dict.get('startASAP', None)

        return cls(day_of_week, hour_of_day, number_of_deferrals, start_asap)

    def as_dict(self) -> dict:
        """
        Returns the Maintenance object as a serializable dict

        Converts enums
        Returns:

        """
        out_dict = self.__dict__
        if type(out_dict['dayOfWeek']) == Weekdays:
            out_dict['dayOfWeek'] = out_dict['dayOfWeek'].value
        return out_dict

    def as_update_dict(self) -> dict:
        """
        Returns a dict with immutable properties removed.
        Returns: dict
        """
        update_dict = self.as_dict()
        del update_dict['numberOfDeferrals']
        for key, val in update_dict.items():
            if val is None:
                del update_dict[key]


        return update_dict



from typing import Optional, List


class TeamRoles:
    def __init__(self, team_id: str, roles: List[str]):
        """Contains a team_id and its associated atlas access roles.

        Args:
            team_id (str):
            roles (List[str]): A list of Atlas access roles.
        """
        self.team_id = team_id
        self.roles = roles


class Team:
    def __init__(self, name: str, id: str = None, usernames: Optional[List[str]] = None, org_id: Optional[str] = None,
                 links: Optional[list] = None):
        """

        Args:
            org_id (str):  The unique identifier for the organization you want to associate the team with.
            name (str): the name of the team
            id (str): The unique identifier for the team.
            links (Optional[list]): Links to team related resources
            usernames Optional[List[str]]:  Valid email addresses of users to add to the new team. Atlas checks whether the user's email belongs to the organization, so that Atlas can properly associate the users to the teams.
        """
        self.org_id = org_id
        self.usernames = usernames
        self.links: Optional[list] = links
        self.id: Optional[str] = id
        self.name: str = name

    @classmethod
    def for_create(cls, org_id: str, name: str, usernames: List[str]):
        """
        Creates a Team object in the format needed to create in the Atlas API
        Args:
            usernames (List[str]):  Valid email addresses of users to add to the new team. Atlas checks whether the user's email belongs to the organization, so that Atlas can properly associate the users to the teams.
            org_id (str):  The unique identifier for the organization you want to associate the team with.
            name (str): the name of the team


        Returns: None

        """
        return cls(name=name, org_id=org_id, usernames=usernames)

    @property
    def as_create_dict(self) -> dict:
        """
        A dictionary in the format the Atlas API expects for Teams
        Returns:

        """
        return dict(name=self.name, usernames=self.usernames)


class TeamRoles:
    def __init__(self, team_id: str, roles: List[str]):
        self.team_id = team_id
        self.roles = roles


# Copyright (c) 2021 Matthew G. Monteleone
# 
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
# 
#     http://www.apache.org/licenses/LICENSE-2.0
# 
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""
Settings module

Provides few constants, APIs endpoints.
"""
import os
from os import getenv


class Settings:
    # Atlas APIs
    BASE_URL = getenv('BASE_URL', 'https://cloud.mongodb.com')
    URI_STUB = '/api/atlas/v1.0'

    api_resources = {
        "Project": {
            "Get One Project": URI_STUB + "/groups/{GROUP_ID}"
        },
        "Monitoring and Logs": {
            "Get all processes for group": "/api/atlas/v1.0/groups/{group_id}/processes?pageNum={"
                                           "page_num}&itemsPerPage={items_per_page}",
            "Get information for process in group": "/api/atlas/v1.0/groups/%s/processes/%s:&s?pageNum=%d"
                                                    "&itemsPerPage=%d",
            "Get measurement for host": "/api/atlas/v1.0/groups/{group_id}/processes/{host}:{"
                                        "port}/measurements?granularity={granularity}&period={period}&m={measurement}",
            "Get list of databases for host": "/api/atlas/v1.0/groups/{GROUP-ID}/processes/{HOST}:{PORT}/databases",
            "Get measurements of database for host.": "/api/atlas/v1.0/groups/{GROUP-ID}/processes/{HOST}:{"
                                                      "PORT}/databases/{DATABASE-NAME}/measurements",
            "Get list of disks or partitions for host.": "/api/atlas/v1.0/groups/{GROUP-ID}/processes/{HOST}:{"
                                                         "PORT}/disks",
            "Get measurements of for host": "/api/atlas/v1.0/groups/{GROUP-ID}/processes/{HOST}:{PORT}/disks/{"
                                            "DISK-NAME}/measurements",
            "Get the log file for a host in the cluster": "/api/atlas/v1.0/groups/{group_id}/clusters/{"
                                                          "host}/logs/{logname}",
            "Get Available Disks for Process": "/api/atlas/v1.0/groups/{group_id}/processes/"
                                               "{host}:{port}/disks",
            "Get Measurements of a Disk for Process": "/api/atlas/v1.0/groups/{group_id}/processes/{host}:{port}/disks/"
                                                      "{disk_name}/measurements",
            "Get Measurements of a Database for Process": "/api/atlas/v1.0/groups/{group_id}/processes/{host}:{port}/"
                                                          "databases/{database_name}/measurements",
            "Get Available Databases for Process": "/api/atlas/v1.0/groups/{group_id}/processes/"
                                                   "{host}:{port}/databases"
        },
        "Events": {
            "Get All Project Events": URI_STUB + "/groups/{group_id}/events??includeRaw=true&pageNum={page_num}"
                                                 "&itemsPerPage={items_per_page}",
            "Get Project Events Since Date": URI_STUB + "/groups/{group_id}/events?includeRaw=true&pageNum={"
                                                        "page_num}&itemsPerPage={items_per_page}&minDate={"
                                                        "min_date}"
        },
        "Clusters": {
            "Get All Clusters": "/api/atlas/v1.0/groups/%s/clusters?pageNum=%d&itemsPerPage=%d",
            "Get a Single Cluster": "/api/atlas/v1.0/groups/%s/clusters/%s",
            "Delete a Cluster": "/api/atlas/v1.0/groups/%s/clusters/%s",
            "Create a Cluster": "/api/atlas/v1.0/groups/{GROUP_ID}/clusters/",
            "Modify a Cluster": "/api/atlas/v1.0/groups/{GROUP_ID}/clusters/{CLUSTER_NAME}",
            "Test Failover": "/api/atlas/v1.0/groups/{GROUP_ID}/clusters/{CLUSTER_NAME}/restartPrimaries",
            "Advanced Configuration Options": "/api/atlas/v1.0/groups/{GROUP_ID}/clusters/{"
                                              "CLUSTER_NAME}/processArgs",

        },
        "Database Users": {
            "Get All Database Users": "/api/atlas/v1.0/groups/%s/databaseUsers?pageNum=%d&itemsPerPage=%d",
            "Get a Single Database User": "/api/atlas/v1.0/groups/%s/databaseUsers/admin/%s",
            "Create a Database User": "/api/atlas/v1.0/groups/%s/databaseUsers",
            "Update a Database User": "/api/atlas/v1.0/groups/%s/databaseUsers/admin/%s",
            "Delete a Database User": "/api/atlas/v1.0/groups/%s/databaseUsers/admin/%s"
        },
        "Alerts": {
            "Get All Alerts": "/api/atlas/v1.0/groups/%s/alerts?pageNum=%d&itemsPerPage=%d",
            "Get All Alerts with status": "/api/atlas/v1.0/groups/%s/alerts?status=%s&pageNum=%d&itemsPerPage=%d",
            "Get an Alert": "/api/atlas/v1.0/groups/%s/alerts/%s",
            "Acknowledge an Alert": "/api/atlas/v1.0/groups/%s/alerts/%s"
        },
        "Whitelist": {
            "Get All Whitelist Entries": "/api/atlas/v1.0/groups/%s/whitelist?pageNum=%d&itemsPerPage=%d",
            "Get Whitelist Entry": "/api/atlas/v1.0/groups/%s/whitelist/%s",
            "Create Whitelist Entry": "/api/atlas/v1.0/groups/%s/whitelist",
            "Delete Whitelist Entry": "/api/atlas/v1.0/groups/%s/whitelist/%s"
        },
        "Maintenance Windows": {
            "Get Maintenance Window": "/api/atlas/v1.0/groups/{GROUP_ID}/maintenanceWindow",
            "Update Maintenance Window": "/api/atlas/v1.0/groups/{GROUP_ID}/maintenanceWindow",
            "Defer Maintenance Window": "/api/atlas/v1.0/groups/{GROUP_ID}/maintenanceWindow/defer",
            "Delete Maintenance Window": "/api/atlas/v1.0/groups/{GROUP_ID}/maintenanceWindow"
        },
        "Organization API Keys": {
            "Get all Organization API Keys associated with org": URI_STUB + "/orgs/{GROUP_ID}/apiKeys",
            "Get one Organization API Key": URI_STUB + "/orgs/{ORG_ID}/apiKeys/{API_KEY_ID}",
            "Get Whitelists for API Key": URI_STUB + "/orgs/{ORG_ID}/apiKeys/{API_KEY_ID}/whitelist",
            "Create one or more whitelist entries for APi Key": URI_STUB + "/orgs/{GROUP_ID}/apiKeys/{"
                                                                           "API_KEY_ID}/whitelist",
            "Get a single whitelist entry": URI_STUB + "/orgs/{GROUP_ID}/apiKeys/{API_KEY_ID}/whitelist/{IP_ADDRESS}"
            # Incomplete
        },
        "Project API Keys": {
            "Get All API Keys Assigned to Project": URI_STUB + "/groups/{GROUP_ID}/apiKeys",

        },
        "Cloud Backup": {
            "Get all Cloud Backups for cluster": URI_STUB + "/groups/{GROUP_ID}/clusters/"
                                                            "{CLUSTER_NAME}/backup/snapshots",
            "Get snapshot by SNAPSHOT-ID": URI_STUB + "/groups/{GROUP_ID}/clusters/"
                                                      "{CLUSTER_NAME}/backup/snapshots/{SNAPSHOT_ID}",
            "Delete snapshot by SNAPSHOT-ID": URI_STUB + "/groups/{GROUP_ID}/clusters/"
                                                         "{CLUSTER_NAME}/backup/snapshots/{SNAPSHOT_ID}",
            "Take an on-demand snapshot": URI_STUB + "/groups/{GROUP_ID}/clusters/{CLUSTER_NAME}/backup/snapshots",

        },
        "Cloud Backup Restore Jobs": {
            "Get all Cloud Backup restore jobs by cluster": URI_STUB + "/groups/{GROUP_ID}/clusters/"
                                                                       "{CLUSTER_NAME}/backup/restoreJobs",
            "Get Cloud Backup restore job by cluster": URI_STUB + "/groups/{GROUP_ID}/clusters/"
                                                                  "{CLUSTER_NAME}/backup/restoreJobs/{JOB_ID}",
            "Restore snapshot by cluster": URI_STUB + "/groups/{GROUP_ID}/clusters/{CLUSTER_NAME}"
                                                      "/backup/restoreJobs",
            "Cancel manual download restore job by job_id": URI_STUB + "/groups/{GROUP_ID}/clusters/{CLUSTER_NAME}"
                                                                       "/backup/restoreJobs/{JOB_ID}"

        }
        ,
        "Projects": {
            "Projects that the authenticated user can access": URI_STUB + "/groups/",
            "Project by group_id": URI_STUB + "/groups/{GROUP_ID}",
            "Project by group name": URI_STUB + "/groups/byName/{GROUP_NAME}",
            "Project teams by group_id": URI_STUB + "/groups/{GROUP_ID}/teams/",
            "Remove the specified Atlas team from the specified project.": URI_STUB + "/groups/{GROUP_ID}/"
                                                                                      "teams/{TEAM_ID}",
            "Atlas Users assigned to project": URI_STUB + "/groups/{GROUP_ID}/users/",
            "Remove Atlas Users assigned to project": URI_STUB + "/groups/{GROUP_ID}/users/{USER_ID}",
            "Pending invitations to the project associated ": URI_STUB + "/groups/{GROUP_ID}/invites",
            "One Pending invitation to the project associated": URI_STUB + "/groups/{GROUP_ID}/"
                                                                           "invites{INVITATION_ID}",
            "Settings for project": URI_STUB + "/groups/{GROUP_ID}/settings",
        }

        ,
        "Organizations": {
            "Orgs the authenticated user can access": URI_STUB + "/orgs/",
            "Org by org_id": URI_STUB + "/orgs/{ORG_ID}",
            "Atlas Users associated to Org": URI_STUB + "/orgs/{ORGS_ID}/users/",
            "Projects associated with the Org": URI_STUB + "/orgs/{ORG_ID}/groups"
        }

    }
    #

    # Atlas enforced
    databaseName = "admin"

    # Atlas default pagination
    pageNum = 1
    itemsPerPage: int = int(os.getenv('ITEMS_PER_PAGE', 500))
    itemsPerPageMin: int = int(os.getenv('ITEMS_PER_PAGE_MIN', 1))
    itemsPerPageMax: int = int(os.getenv('ITEMS_PER_PAGE_MAX', 2000))

    # Requests
    requests_timeout = 10
    file_request_timeout = 360

    # HTTP Return code
    SUCCESS = 200
    CREATED = 201
    ACCEPTED = 202
    NO_CONTENT = 204
    BAD_REQUEST = 400
    UNAUTHORIZED = 401
    FORBIDDEN = 403
    NOTFOUND = 404
    METHOD_NOT_ALLOWED = 405
    CONFLICT = 409
    SERVER_ERRORS = 500



from __future__ import division, absolute_import, print_function, unicode_literals

from datetime import datetime
from statistics import mean, StatisticsError
from typing import Optional, Tuple, List, Iterable, NewType
import logging
import humanfriendly as hf
from dateutil.parser import parse

from atlasapi.atlas_types import OptionalFloat
from atlasapi.lib import _GetAll, AtlasPeriods, AtlasGranularities

logger: logging.Logger = logging.getLogger('Atlas.measurements')


class StatisticalValues:
    def __init__(self, data_list: list):
        try:
            self.samples: int = len(clean_list(data_list))
            self.mean: float = float(mean(clean_list(data_list)))
            self.min: float = float(min(clean_list(data_list)))
            self.max: float = float(max(clean_list(data_list)))
        except StatisticsError:
            logger.warning('Could not compute statistical values.')
            self.samples: int = 0
            self.mean: float = 0
            self.min: float = 0
            self.max: float = 0


class StatisticalValuesFriendly:
    def __init__(self, data_list: list, data_type: str = None) -> None:
        """Returns human-readable values for stats

        Args:
            data_list:
            data_type: The datatype either bytes or number
        """
        try:
            if data_type is None:
                data_type = 'SCALAR_PER_SECOND'
            if data_type == 'BYTES':
                self.mean: str = hf.format_size(mean(clean_list(data_list)))
                self.min: str = hf.format_size(min(clean_list(data_list)))
                self.max: str = hf.format_size(max(clean_list(data_list)))
            elif data_type == 'SCALAR':
                self.mean: str = hf.format_number(mean(clean_list(data_list)))
                self.min: str = hf.format_number(min(clean_list(data_list)))
                self.max: str = hf.format_number(max(clean_list(data_list)))
            else:
                self.mean: str = hf.format_number(mean(clean_list(data_list)))
                self.min: str = hf.format_number(min(clean_list(data_list)))
                self.max: str = hf.format_number(max(clean_list(data_list)))

        except StatisticsError:
            logger.warning('Could not compute statistical values.')
            self.mean: str = 'No Value'
            self.min: str = 'No value'
            self.max: str = 'No value'


class AtlasMeasurementTypes(_GetAll):
    """
    Helper class for all available atlas measurements.

    All classes and embedded classes have a get_all class method that returns an iterator of all measurements
    and sub measurements.

    """
    connections = 'CONNECTIONS'

    class Asserts(_GetAll):
        regular = 'ASSERT_REGULAR'
        warning = 'ASSERT_WARNING'
        msg = 'ASSERT_MSG'
        user = 'ASSERT_USER'

    class Cache(_GetAll):
        bytes_read = 'CACHE_BYTES_READ_INTO'
        bytes_written = 'CACHE_BYTES_WRITTEN_FROM'
        dirty = 'CACHE_DIRTY_BYTES'
        used = 'CACHE_USED_BYTES'

    class Cursors(_GetAll):
        open = 'CURSORS_TOTAL_OPEN'
        timed_out = 'CURSORS_TOTAL_TIMED_OUT'

    class Db(_GetAll):
        storage = 'DB_STORAGE_TOTAL'
        data_size = 'DB_DATA_SIZE_TOTAL'

    class DocumentMetrics(_GetAll):
        returned = 'DOCUMENT_METRICS_RETURNED'
        inserted = 'DOCUMENT_METRICS_INSERTED'
        updated = 'DOCUMENT_METRICS_UPDATED'
        deleted = 'DOCUMENT_METRICS_DELETED'

    class ExtraInfo(_GetAll):
        page_faults = 'EXTRA_INFO_PAGE_FAULTS'

    class GlobalLockCurrentQueue(_GetAll):
        total = 'GLOBAL_LOCK_CURRENT_QUEUE_TOTAL'
        readers = 'GLOBAL_LOCK_CURRENT_QUEUE_READERS'
        writers = 'GLOBAL_LOCK_CURRENT_QUEUE_WRITERS'

    class Memory(_GetAll):
        resident = 'MEMORY_RESIDENT'
        virtual = 'MEMORY_VIRTUAL'
        mapped = 'MEMORY_MAPPED'

    class Network(_GetAll):
        bytes_id = 'NETWORK_BYTES_IN'  # initial typo, kept for backwards compatibility
        bytes_in = 'NETWORK_BYTES_IN'
        bytes_out = 'NETWORK_BYTES_OUT'
        num_requests = 'NETWORK_NUM_REQUESTS'

    class Opcounter(_GetAll):
        cmd = 'OPCOUNTER_CMD'
        query = 'OPCOUNTER_QUERY'
        update = 'OPCOUNTER_UPDATE'
        delete = 'OPCOUNTER_DELETE'
        getmore = 'OPCOUNTER_GETMORE'
        insert = 'OPCOUNTER_INSERT'

        class Repl(_GetAll):
            cmd = 'OPCOUNTER_REPL_CMD'
            update = 'OPCOUNTER_REPL_UPDATE'
            delete = 'OPCOUNTER_REPL_DELETE'
            insert = 'OPCOUNTER_REPL_INSERT'

    class Operations(_GetAll):
        scan_and_order = 'OPERATIONS_SCAN_AND_ORDER'

        class ExecutionTime(_GetAll):
            reads = 'OP_EXECUTION_TIME_READS'
            writes = 'OP_EXECUTION_TIME_WRITES'
            commands = 'OP_EXECUTION_TIME_COMMANDS'

    class Oplog(_GetAll):
        master_time = 'OPLOG_MASTER_TIME'
        rate = 'OPLOG_RATE_GB_PER_HOUR'

    class QueryExecutor(_GetAll):
        scanned = 'QUERY_EXECUTOR_SCANNED'
        scanned_objects = 'QUERY_EXECUTOR_SCANNED_OBJECTS'

    class QueryTargetingScanned(_GetAll):
        per_returned = 'QUERY_TARGETING_SCANNED_PER_RETURNED'
        objects_per_returned = 'QUERY_TARGETING_SCANNED_OBJECTS_PER_RETURNED'

    class TicketsAvailable(_GetAll):
        reads = 'TICKETS_AVAILABLE_READS'
        writes = 'TICKETS_AVAILABLE_WRITE'

    class CPU(_GetAll):
        class Process(_GetAll):
            user = 'PROCESS_CPU_USER'
            kernel = 'PROCESS_CPU_KERNEL'
            children_user = 'PROCESS_CPU_CHILDREN_USER'
            children_kernel = 'PROCESS_CPU_CHILDREN_KERNEL'

        class ProcessNormalized(_GetAll):
            user = 'PROCESS_NORMALIZED_CPU_USER'
            kernel = 'PROCESS_NORMALIZED_CPU_KERNEL'
            children_user = 'PROCESS_NORMALIZED_CPU_CHILDREN_USER'
            children_kernel = 'PROCESS_NORMALIZED_CPU_CHILDREN_KERNEL'

        class System(_GetAll):
            user = 'SYSTEM_CPU_USER'
            kernel = 'SYSTEM_CPU_KERNEL'
            nice = 'SYSTEM_CPU_NICE'
            iowait = 'SYSTEM_CPU_IOWAIT'
            irq = 'SYSTEM_CPU_IRQ'
            softirq = 'SYSTEM_CPU_SOFTIRQ'
            guest = 'SYSTEM_CPU_GUEST'
            steal = 'SYSTEM_CPU_STEAL'

        class SystemNormalized(_GetAll):
            user = 'SYSTEM_NORMALIZED_CPU_USER'
            kernel = 'SYSTEM_NORMALIZED_CPU_KERNEL'
            nice = 'SYSTEM_NORMALIZED_CPU_NICE'
            iowait = 'SYSTEM_NORMALIZED_CPU_IOWAIT'
            irq = 'SYSTEM_NORMALIZED_CPU_IRQ'
            softirq = 'SYSTEM_NORMALIZED_CPU_SOFTIRQ'
            guest = 'SYSTEM_NORMALIZED_CPU_GUEST'
            steal = 'SYSTEM_NORMALIZED_CPU_STEAL'

    class Disk(_GetAll):
        class IOPS(_GetAll):
            read = 'DISK_PARTITION_IOPS_READ'
            read_max = 'MAX_DISK_PARTITION_IOPS_READ'
            write = 'DISK_PARTITION_IOPS_WRITE'
            write_max = 'MAX_DISK_PARTITION_IOPS_WRITE'
            total = 'DISK_PARTITION_IOPS_TOTAL'
            total_max = 'MAX_DISK_PARTITION_IOPS_TOTAL'

        class Util(_GetAll):
            util = 'DISK_PARTITION_UTILIZATION'
            util_max = 'MAX_DISK_PARTITION_UTILIZATION'

        class Latency(_GetAll):
            read = 'DISK_PARTITION_LATENCY_READ'
            read_max = 'MAX_DISK_PARTITION_LATENCY_READ'
            write = 'DISK_PARTITION_LATENCY_WRITE'
            write_max = 'MAX_DISK_PARTITION_LATENCY_WRITE'

        class Free(_GetAll):
            space_free = 'DISK_PARTITION_SPACE_FREE'
            space_free_max = 'MAX_DISK_PARTITION_SPACE_FREE'
            used = 'DISK_PARTITION_SPACE_USED'
            used_max = 'MAX_DISK_PARTITION_SPACE_USED'
            percent_fee = 'DISK_PARTITION_SPACE_PERCENT_FREE'
            percent_free_max = 'MAX_DISK_PARTITION_SPACE_PERCENT_FREE'
            percent_used = 'DISK_PARTITION_SPACE_PERCENT_USED'
            percent_used_max = 'MAX_DISK_PARTITION_SPACE_PERCENT_USED'

    class Namespaces(_GetAll):
        """Metrics regarding namespaces (databases) on each host.

        As found in dbstats (https://www.mongodb.com/docs/manual/reference/command/dbStats/)
            """
        object_size = 'DATABASE_AVERAGE_OBJECT_SIZE' # dbStats.avgObjSize Average size of each document in bytes. This is the dataSize divided by the number of documents. The scale argument does not affect the avgObjSize value.
        collection_count = 'DATABASE_COLLECTION_COUNT'
        data_size = 'DATABASE_DATA_SIZE' # Total size of the uncompressed data held in the database. The dataSize decreases when you remove documents.
        storage_size = 'DATABASE_STORAGE_SIZE' # Sum of the space allocated to all collections in the database for document storage, including free space. storageSize does not include space allocated to indexes. See indexSize for the total index size.
        index_size = 'DATABASE_INDEX_SIZE' # Sum of the space allocated to all indexes in the database, including free index space.
        index_count = 'DATABASE_INDEX_COUNT'
        extent_count = 'DATABASE_EXTENT_COUNT' # ?
        object_count = 'DATABASE_OBJECT_COUNT' # Number of objects (specifically, documents) in the database across all collections.
        view_count = 'DATABASE_VIEW_COUNT'


# noinspection PyBroadException
class AtlasMeasurementValue(object):
    def __init__(self, value_dict: dict):
        """
        Class for holding a measurement value
        :type value_dict: dict
        :param value_dict: An Atlas standard Measurement value dictionary.
        """
        timestamp: int = value_dict.get('timestamp', None)
        value: float = value_dict.get('value', None)
        try:
            self.timestamp: datetime = parse(timestamp)
        except (ValueError, TypeError):
            logger.warning('Could not parse "{}" as a datetime.')
            self.timestamp = None
        try:
            if value is None:
                self.value = None
            self.value: float = float(value)
        except ValueError as e:
            self.value = None
            logger.warning('Could not parse the metric value "{}". Error was {}'.format(value, e))
        except TypeError:
            logger.info('Value is none.')
            self.value = None

    # noinspection PyBroadException
    @property
    def value_int(self) -> Optional[int]:
        try:
            return int(self.value)
        except Exception as e:
            return None

    @property
    def value_float(self) -> Optional[float]:
        try:
            return float(self.value)
        except Exception:
            return None

    def as_dict(self) -> dict:
        return dict(timestamp=self.timestamp.__str__(), value=self.value, value_int=self.value_int,
                    value_float=self.value_float)

    @property
    def as_tuple(self) -> Tuple[datetime, OptionalFloat]:
        """
        Returns a MeasurementValue as a tuple, timestamp first.
        :rtype: Tuple[datetime,OptionalFloat]
        :return: A tuple with a datetime and a float
        """
        return self.timestamp, self.value


class AtlasMeasurement(object):
    """A point in time container for an Atlas measurement.

            For a certain period, granularity and measurementType holds a list fo measurementValues.

            Args:
                name (AtlasMeasurementTypes): The name of the measurement type
                units (Text): Descriptive text of units used.
                period (AtlasPeriods): The period the measurement covers
                granularity (AtlasGranularities): The granularity used for the measurement
                measurements (List[AtlasMeasurementValue]): A list of the actual measurement values
            """

    def __init__(self, name: AtlasMeasurementTypes, period: AtlasPeriods,
                 granularity: AtlasGranularities, units: str = None, measurements: List[AtlasMeasurementValue] = None):
        if measurements is None:
            measurements = list()
        self.name: AtlasMeasurementTypes = name
        self.units: str = units
        self.period: AtlasPeriods = period
        self.granularity: AtlasGranularities = granularity
        self._measurements: List[AtlasMeasurementValue] = measurements

    @property
    def measurements(self) -> Iterable[AtlasMeasurementValue]:
        """
        Getter for the measurements.

        Returns:
            Iterator[AtlasMeasurementValue]: An iterator containing values objects.
        """
        for item in self._measurements:
            yield item

    @measurements.setter
    def measurements(self, value):
        if type(value) == list:
            self._measurements.extend(value)
        else:
            self._measurements.append(value)

    @measurements.deleter
    def measurements(self):
        self._measurements = []

    def measurements_as_tuples(self):
        if isinstance(self._measurements[0], AtlasMeasurementValue):
            for item in self._measurements:
                yield item.as_tuple

    @property
    def date_start(self):
        """The date of the first measurement.

        Returns:
            datetime: The date of the first measurement.
        """
        seq = [x.timestamp for x in self._measurements]
        return min(seq)

    @property
    def date_end(self):
        """The date of the last measurement

        Returns:
            datetime: The date of the last measurement.

        """
        seq = [x.timestamp for x in self._measurements]
        return max(seq)

    @property
    def measurements_count(self):
        """The count of measurements

        Returns:
            int: The count of measurements in the set
        """
        return len(self._measurements)

    @property
    def as_dict(self):
        """Returns the measurement as a dict, including the computed properties.

        Returns:
            dict:
        """
        return dict(measurements=self._measurements, date_start=self.date_start, date_end=self.date_end, name=self.name,
                    units=self.units, period=self.period, granularity=self.granularity,
                    measurements_count=self.measurements_count
                    )

    @property
    def measurement_stats(self) -> StatisticalValues:
        """Returns a statistical info for measurement data"""
        data_list = list()
        for each_measurement in self.measurements:
            data_list.append(each_measurement.value_float)
        return StatisticalValues(data_list=data_list)

    @property
    def measurement_stats_friendly(self) -> StatisticalValuesFriendly:
        """Returns  statistical info for measurement data in friendly bytes format"""
        data_list = list()
        for each_measurement in self.measurements:
            data_list.append(each_measurement.value_float)
        return StatisticalValuesFriendly(data_list=data_list, data_type=self.units)

    def __hash__(self):
        return hash(self.name + '-' + self.period)

    def __eq__(self, other):
        """
        Measurements are considered duplicate of name and period are the same
        :param other:
        :return:
        """
        if isinstance(other, AtlasMeasurement):
            return (self.name == other.name) and (self.period == other.period)


ListOfAtlasMeasurementValues = NewType('ListOfAtlasMeasurementValues', List[Optional[AtlasMeasurementValue]])
OptionalAtlasMeasurement = NewType('OptionalAtlasMeasurement', Optional[AtlasMeasurement])


def clean_list(data_list: list) -> list:
    """Returns a list with any none values removed

    Args:
        data_list (list): The list to be cleaned

    Returns (list): The list cleaned of None values.

    """
    return list(filter(None, data_list))



# Copyright (c) 2022 Matthew G. Monteleone
# 
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
# 
#     http://www.apache.org/licenses/LICENSE-2.0
# 
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""
Errors module

Provides all specific Exceptions
"""

from atlasapi.settings import Settings
from pprint import pprint
import logging

logger = logging.getLogger('Error_Handler')

class ErrRole(Exception):
    """A role is not compatible with Atlas"""
    pass


class ErrPagination(Exception):
    """An issue occurs during a "Get All" function"""

    def __init__(self):
        super().__init__("Issue occurs during the pagination.")


class ErrPaginationLimits(Exception):
    """Out of limit on 'pageNum' or 'itemsPerPage' parameters

    Constructor

    Args:
        error_code (int): ERR_PAGE_NUM or ERR_ITEMS_PER_PAGE
    """
    ERR_PAGE_NUM = 0
    ERR_ITEMS_PER_PAGE = 1

    def __init__(self, error_code):
        if error_code == ErrPaginationLimits.ERR_PAGE_NUM:
            super().__init__("pageNum can't be smaller than 1")
        elif error_code == ErrPaginationLimits.ERR_ITEMS_PER_PAGE:
            super().__init__("itemsPerPage can't be smaller than %d and greater than %d" % (
                Settings.itemsPerPageMin, Settings.itemsPerPageMax))
        else:
            super().__init__(str(error_code))

    def checkAndRaise(pageNum, itemsPerPage):
        """Check and Raise an Exception if needed

        Args:
            pageNum (int): Page number
            itemsPerPage (int): Number of items per Page

        Raises:
            ErrPaginationLimits: If we are out of limits

        """
        if pageNum < 1:
            raise ErrPaginationLimits(ErrPaginationLimits.ERR_PAGE_NUM)

        if itemsPerPage < Settings.itemsPerPageMin or itemsPerPage > Settings.itemsPerPageMax:
            raise ErrPaginationLimits(ErrPaginationLimits.ERR_ITEMS_PER_PAGE)


class ErrAtlasGeneric(Exception):
    """Atlas Generic Exception

    Constructor

    Args:
        msg (str): Short description of the error
        c (int): HTTP code
        details (dict): Response payload
    """

    def __init__(self, msg, c, details):
        super().__init__(msg)
        self.code = c
        self.details = details

    def getAtlasResponse(self):
        """Get details about the Atlas response

        Returns:
            int, str: HTTP code, Response payload

        """
        return self.code, self.details

class ErrMaintenanceError(ErrAtlasGeneric):
    """Atlas : Atlas MaintenanceRelatedError

    Constructor

    Args:
        c (int): HTTP code
        details (dict): Response payload
    """

    def __init__(self, c, details):
        super().__init__(details.get('detail', f'Atlas Maintenance Error: {details.get("detail")} '), c, details)


class ErrAtlasBadRequest(ErrAtlasGeneric):
    """Atlas : Bad Request

    Constructor

    Args:
        c (int): HTTP code
        details (dict): Response payload
    """

    def __init__(self, c, details):

        if details.get('errorCode', None) == 'DUPLICATE_CLUSTER_NAME':
            raise (ErrAtlasDuplicateClusterName(c, details))
        if details.get('errorCode', None) == 'RESOURCE_NOT_FOUND_FOR_JOB':
            raise (ErrAtlasJobError(c, details))
        if details.get('errorCode', None) == 'CANNOT_CANCEL_AUTOMATED_RESTORE':
            raise (ErrAtlasBackupError(c,details))
        if details.get('errorCode', None) in ['ATLAS_MAINTENANCE_ALREADY_SCHEDULED',
                                              'ATLAS_NUM_MAINTENANCE_DEFERRALS_EXCEEDED']:
            raise (ErrMaintenanceError(c,details))
        else:
            logger.critical(f"A generic error was raised")
            logger.critical(details)


        super().__init__("Something was wrong with the client request.", c, details)


class ErrAtlasJobError(ErrAtlasGeneric):
    """Atlas : Job error Clustername

    Constructor

    Args:
        c (int): HTTP code
        details (dict): Response payload
    """

    def __init__(self, c, details):
        super().__init__(details.get('detail', 'Duplicate Error'), c, details)


class ErrAtlasDuplicateClusterName(ErrAtlasGeneric):
    """Atlas : Duplicate Clustername

    Constructor

    Args:
        c (int): HTTP code
        details (dict): Response payload
    """

    def __init__(self, c, details):
        super().__init__(details.get('detail', 'Duplicate Error'), c, details)


class ErrAtlasBackupError(ErrAtlasGeneric):
    """Atlas : Atlas Backup

    Constructor

    Args:
        c (int): HTTP code
        details (dict): Response payload
    """

    def __init__(self, c, details):
        super().__init__(details.get('detail', 'Atlas Backup error'), c, details)


class ErrAtlasUnauthorized(ErrAtlasGeneric):
    """Atlas : Unauthorized

    Constructor

    Args:
        c (int): HTTP code
        details (dict): Response payload
    """

    def __init__(self, c, details):
        super().__init__("Authentication is required", c, details)


class ErrAtlasForbidden(ErrAtlasGeneric):
    """Atlas : Forbidden

    Constructor

    Args:
        c (int): HTTP code
        details (dict): Response payload
    """

    def __init__(self, c, details):
        if details['errorCode'] == 'ORG_REQUIRES_WHITELIST':
            raise ErrAtlasForbiddenWL(c, details)
        else:
            super().__init__("Access to the specified resource is not permitted.", c, details)


class ErrAtlasForbiddenWL(ErrAtlasGeneric):
    """Atlas : Forbidden by WhiteList

    Constructor

    Args:
        c (int): HTTP code
        details (dict): Response payload
    """

    def __init__(self, c, details):
        super().__init__("This organization requires access through a whitelist of ip ranges.", c, details)


class ErrAtlasNotFound(ErrAtlasGeneric):
    """Atlas : Not Found

    Constructor

    Args:
        c (int): HTTP code
        details (dict): Response payload
    """

    def __init__(self, c, details):
        super().__init__("The requested resource does not exist.", c, details)


class ErrAtlasMethodNotAllowed(ErrAtlasGeneric):
    """Atlas : Method Not Allowed

    Constructor

    Args:
        c (int): HTTP code
        details (dict): Response payload
    """

    def __init__(self, c, details):
        super().__init__("The HTTP method is not supported for the specified resource.", c, details)


class ErrAtlasConflict(ErrAtlasGeneric):
    """Atlas : Conflict

    Constructor

    Args:
        c (int): HTTP code
        details (dict): Response payload
    """

    def __init__(self, c, details):
        super().__init__(
            "This is typically the response to a request to create or modify a property of an entity that is unique "
            "when an existing entity already exists with the same value for that property.",
            c, details)


class ErrAtlasRestoreConflictError(ErrAtlasGeneric):
    """Atlas : RestoreConflictError

    Constructor

    Args:
        c (int): HTTP code
        details (dict): Response payload
    """

    def __init__(self, c, details):
        super().__init__(
            "This is an error when there are issues with cluster state during restore which prevents API actions",
            c, details)


class ErrAtlasServerErrors(ErrAtlasGeneric):
    """Atlas : Server Errors

    Constructor

    Args:
        c (int): HTTP code
        details (dict): Response payload
    """

    def __init__(self, c, details):
        super().__init__("Something unexpected went wrong.", c, details)
        pprint(details)


class ErrConfirmationRequested(Exception):
    """No Confirmation provided

    Constructor

    Args:
        msg (str): Short description of the error
    """

    def __init__(self, msg):
        super().__init__(msg)



# Copyright (c) 2019 Matthew G. Monteleone
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
# Modifications copyright (C) 2019 Matthew Monteleone

from enum import Enum
from typing import List, NewType, Optional, Union
from pprint import pprint
from datetime import datetime
import pytz
import uuid

FORMAT = '%Y-%m-%dT%H:%M:%SZ'


class Alert(object):
    def __init__(self, data_dict: dict):
        self.alertConfigId: str = data_dict.get('alertConfigId', None)
        try:
            self.alertConfigId: Union[datetime, None] = datetime.strptime(data_dict.get("created", None),
                                                                          FORMAT).astimezone(tz=pytz.UTC)
        except ValueError:
            self.alertConfigId: Union[datetime, None] = None

        try:
            self.lastNotified: Union[datetime, None] = datetime.strptime(data_dict.get("lastNotified", None),
                                                                         FORMAT).astimezone(tz=pytz.UTC)
        except ValueError:
            self.lastNotified: Union[datetime, None] = None

        try:
            self.resolved: Union[datetime, None] = datetime.strptime(data_dict.get("resolved", None),
                                                                     FORMAT).astimezone(tz=pytz.UTC)
        except ValueError:
            self.resolved: Union[datetime, None] = None

        try:
            self.updated: Union[datetime, None] = datetime.strptime(data_dict.get("updated", None),
                                                                    FORMAT).astimezone(tz=pytz.UTC)
        except ValueError:
            self.updated: Union[datetime, None] = None

        self.currentValue: dict = data_dict.get('currentValue', None)
        self.eventTypeName: str = data_dict.get('eventTypeName', None)
        self.groupId: str = data_dict.get('groupId', None)
        self.hostnameAndPort: str = data_dict.get('hostnameAndPort', None)
        self.id: str = data_dict.get('id', None)
        self.links: list = data_dict.get('links', None)
        self.metricName: str = data_dict.get('metricName', None)
        self.replicaSetName: str = data_dict.get('replicaSetName', None)
        self.status: str = data_dict.get('status', None)
        self.typeName: str = data_dict.get('typeName', None)
